@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import copy
 import hashlib
 import io
@@ -9,13 +6,13 @@ import re
 import struct
 import weakref
 from collections import namedtuple
-from urllib import parse as urlparse
 
 import numpy as np
 
 import yaml
 
 from . import compression as mcompression
+from .config import get_config
 from .compat.numpycompat import NUMPY_LT_1_7
 from . import constants
 from . import generic_io
@@ -23,6 +20,8 @@ from . import treeutil
 from . import util
 from . import yamlutil
 
+from .util import patched_urllib_parse
+
 
 class BlockManager:
     """
@@ -49,6 +48,7 @@ class BlockManager:
         self._memmap = not copy_arrays
         self._lazy_load = lazy_load
         self._readonly = readonly
+        self._internal_blocks_mapped = False
 
     def __len__(self):
         """
@@ -65,6 +65,15 @@ class BlockManager:
         """
         Add an internal block to the manager.
         """
+        if not self._internal_blocks_mapped:
+            # If the block index is missing we need to locate the remaining
+            # blocks so that we don't accidentally add our new block
+            # in the middle of the list.
+            self.finish_reading_internal_blocks()
+
+        self._add(block)
+
+    def _add(self, block):
         block_set = self._block_type_mapping.get(block.array_storage, None)
         if block_set is not None:
             if block not in block_set:
@@ -128,6 +137,7 @@ class BlockManager:
             self.add(block)
             if array_storage == 'streamed':
                 block.output_compression = None
+                block.output_compression_kwargs = None
 
     @property
     def blocks(self):
@@ -227,7 +237,7 @@ class BlockManager:
             fd, past_magic=past_magic,
             validate_checksum=self._validate_checksums)
         if block is not None:
-            self.add(block)
+            self._add(block)
 
         return block
 
@@ -291,6 +301,8 @@ class BlockManager:
                 if last_block is None:
                     break
 
+        self._internal_blocks_mapped = True
+
     def write_internal_blocks_serial(self, fd, pad_blocks=False):
         """
         Write all blocks to disk serially.
@@ -456,7 +468,7 @@ class BlockManager:
         # Read blocks in reverse order from the end of the file
         while True:
             # Look for the index header
-            idx = content.find(constants.INDEX_HEADER)
+            idx = content.rfind(constants.INDEX_HEADER)
             if idx != -1:
                 content = content[idx:]
                 index_start = block_start + idx
@@ -532,6 +544,10 @@ class BlockManager:
         # We already read the last block in the file -- no need to read it again
         self._internal_blocks.append(block)
 
+        # Record that all block locations have been mapped out (used to avoid
+        # unnecessary calls to finish_reading_internal_blocks later).
+        self._internal_blocks_mapped = True
+
         # Materialize the internal blocks if we are not lazy
         if not self.lazy_load:
             self.finish_reading_internal_blocks()
@@ -551,13 +567,13 @@ class BlockManager:
         """
         if uri is None:
             uri = ''
-        parts = list(urlparse.urlparse(uri))
+        parts = list(patched_urllib_parse.urlparse(uri))
         path = parts[2]
         dirname, filename = os.path.split(path)
         filename = self.get_external_filename(filename, index)
         path = os.path.join(dirname, filename)
         parts[2] = path
-        return urlparse.urlunparse(parts)
+        return patched_urllib_parse.urlunparse(parts)
 
     def _find_used_blocks(self, tree, ctx):
         reserved_blocks = set()
@@ -580,17 +596,20 @@ class BlockManager:
             self.set_array_storage(block, all_array_storage)
 
         all_array_compression = getattr(ctx, '_all_array_compression', 'input')
+        all_array_compression_kwargs = getattr(ctx, '_all_array_compression_kwargs', {})
         # Only override block compression algorithm if it wasn't explicitly set
         # by AsdfFile.set_array_compression.
         if all_array_compression != 'input':
             block.output_compression = all_array_compression
+            block.output_compression_kwargs = all_array_compression_kwargs
 
-        auto_inline = getattr(ctx, '_auto_inline', None)
-        if auto_inline and block.array_storage in ['internal', 'inline']:
-            if np.product(block.data.shape) < auto_inline:
-                self.set_array_storage(block, 'inline')
-            else:
-                self.set_array_storage(block, 'internal')
+        if all_array_storage is None:
+            threshold = get_config().array_inline_threshold
+            if threshold is not None and block.array_storage in ['internal', 'inline']:
+                if np.product(block.data.shape) < threshold:
+                    self.set_array_storage(block, 'inline')
+                else:
+                    self.set_array_storage(block, 'internal')
 
     def finalize(self, ctx):
         """
@@ -670,8 +689,7 @@ class BlockManager:
             raise ValueError("Block '{0}' not found.".format(source))
 
         elif isinstance(source, str):
-            asdffile = self._asdffile().open_external(
-                source, do_not_fill_defaults=True)
+            asdffile = self._asdffile().open_external(source)
             block = asdffile.blocks._internal_blocks[0]
             self.set_array_storage(block, 'external')
 
@@ -711,7 +729,7 @@ class BlockManager:
                         "Can't write external blocks, since URI of main file is "
                         "unknown.")
 
-                parts = list(urlparse.urlparse(self._asdffile().uri))
+                parts = list(patched_urllib_parse.urlparse(self._asdffile().uri))
                 path = parts[2]
                 filename = os.path.basename(path)
                 return self.get_external_filename(filename, i)
@@ -769,6 +787,25 @@ class BlockManager:
         self.add(block)
         return block
 
+    def get_output_compressions(self):
+        '''
+        Get the list of unqiue compressions used on blocks.
+        '''
+        return list(set([b.output_compression for b in self.blocks]))
+
+    def get_output_compression_extensions(self):
+        '''
+        Infer the compression extensions used on blocks.
+        Note that this is somewhat indirect and could be fooled if a new extension
+        for the same compression label is loaded after the compression of the block.
+        '''
+        ext = []
+        for label in self.get_output_compressions():
+            compressor = mcompression._get_compressor_from_extensions(label, return_extension=True)
+            if compressor != None:
+                ext += [compressor[1]]  # second item is the extension
+        return ext
+
     def __getitem__(self, arr):
         return self.find_or_create_block_for_array(arr, object())
 
@@ -803,6 +840,7 @@ class Block:
         self._offset = None
         self._input_compression = None
         self._output_compression = 'input'
+        self._output_compression_kwargs = {}
         self._checksum = None
         self._should_memmap = memmap
         self._memmapped = False
@@ -854,13 +892,6 @@ class Block:
         """
         return self.offset + self.header_size + self.allocated
 
-    def override_byteorder(self, byteorder):
-        """
-        Hook to permit overriding the byteorder value stored in the
-        tree.  This is used to support blocks stored in FITS files.
-        """
-        return byteorder
-
     @property
     def trust_data_dtype(self):
         """
@@ -899,6 +930,21 @@ class Block:
     def output_compression(self, compression):
         self._output_compression = mcompression.validate(compression)
 
+    @property
+    def output_compression_kwargs(self):
+        """
+        The configuration options to the Compressor constructur
+        used to write the block.
+        :return:
+        """
+        return self._output_compression_kwargs
+
+    @output_compression_kwargs.setter
+    def output_compression_kwargs(self, config):
+        if config is None:
+            config = {}
+        self._output_compression_kwargs = config.copy()
+
     @property
     def checksum(self):
         return self._checksum
@@ -913,11 +959,11 @@ class Block:
         else:
             self._checksum = checksum
 
-    def _calculate_checksum(self, data):
+    def _calculate_checksum(self, array):
         # The following line is safe because we're only using
         # the MD5 as a checksum.
         m = hashlib.new('md5') # nosec
-        m.update(self.data.ravel('K'))
+        m.update(array)
         return m.digest()
 
     def validate_checksum(self):
@@ -932,7 +978,7 @@ class Block:
             `False`.
         """
         if self._checksum:
-            checksum = self._calculate_checksum(self.data)
+            checksum = self._calculate_checksum(self._flattened_data)
             if checksum != self._checksum:
                 return False
         return True
@@ -941,7 +987,7 @@ class Block:
         """
         Update the checksum based on the current data contents.
         """
-        self._checksum = self._calculate_checksum(self.data)
+        self._checksum = self._calculate_checksum(self._flattened_data)
 
     def update_size(self):
         """
@@ -950,13 +996,15 @@ class Block:
         updating the file in-place, otherwise the work is redundant.
         """
         if self._data is not None:
-            self._data_size = self._data.data.nbytes
+            data = self._flattened_data
+            self._data_size = data.nbytes
 
             if not self.output_compression:
                 self._size = self._data_size
             else:
                 self._size = mcompression.get_compressed_size(
-                    self._data, self.output_compression)
+                    data, self.output_compression,
+                    config=self.output_compression_kwargs)
         else:
             self._data_size = self._size = 0
 
@@ -1016,9 +1064,13 @@ class Block:
 
         # This is used by the documentation system, but nowhere else.
         self._flags = header['flags']
-        self.input_compression = header['compression']
         self._set_checksum(header['checksum'])
 
+        try:
+            self.input_compression = header['compression']
+        except ValueError as v:
+            raise v  # TODO: hint extension?
+
         if (self.input_compression is None and
                 header['used_size'] != header['data_size']):
             raise ValueError(
@@ -1101,30 +1153,54 @@ class Block:
             self._data = self._fd.memmap_array(self.data_offset, self._size)
             self._memmapped = True
 
+    @property
+    def _flattened_data(self):
+        """
+        Retrieve flattened data suitable for writing.
+
+        Returns
+        -------
+        np.ndarray
+            1D contiguous array.
+        """
+        data = self.data
+
+        # 'K' order flattens the array in the order that elements
+        # occur in memory, except axes with negative strides which
+        # are reversed.  That is a problem for base arrays with
+        # negative strides and is an outstanding bug in this library.
+        return data.ravel(order='K')
+
     def write(self, fd):
         """
         Write an internal block to the given Python file-like object.
         """
         self._header_size = self._header.size
 
+        if self._data is not None:
+            data = self._flattened_data
+        else:
+            data = None
+
         flags = 0
         data_size = used_size = allocated_size = 0
         if self._array_storage == 'streamed':
             flags |= constants.BLOCK_FLAG_STREAMED
-        elif self._data is not None:
-            self.update_checksum()
-            data_size = self._data.nbytes
+        elif data is not None:
+            self._checksum = self._calculate_checksum(data)
+            data_size = data.nbytes
             if not fd.seekable() and self.output_compression:
                 buff = io.BytesIO()
-                mcompression.compress(buff, self._data,
-                                      self.output_compression)
+                mcompression.compress(buff, data,
+                                      self.output_compression,
+                                      config=self.output_compression_kwargs)
                 self.allocated = self._size = buff.tell()
             allocated_size = self.allocated
             used_size = self._size
         self.input_compression = self.output_compression
 
         if allocated_size < used_size:
-            raise RuntimeError("Block used size {} larger than allocated size {}".format(used_size, allocated_size))
+            raise RuntimeError(f"Block used size {used_size} larger than allocated size {allocated_size}")
 
         if self.checksum is not None:
             checksum = self.checksum
@@ -1141,7 +1217,7 @@ class Block:
             used_size=used_size, data_size=data_size,
             checksum=checksum))
 
-        if self._data is not None:
+        if data is not None:
             if self.output_compression:
                 if not fd.seekable():
                     fd.write(buff.getvalue())
@@ -1152,7 +1228,8 @@ class Block:
                     # header.
                     start = fd.tell()
                     mcompression.compress(
-                        fd, self._data, self.output_compression)
+                        fd, data, self.output_compression,
+                        config=self.output_compression_kwargs)
                     end = fd.tell()
                     self.allocated = self._size = end - start
                     fd.seek(self.offset + 6)
@@ -1163,8 +1240,8 @@ class Block:
                     fd.seek(end)
             else:
                 if used_size != data_size:
-                    raise RuntimeError("Block used size {} is not equal to the data size {}".format(used_size, data_size))
-                fd.write_array(self._data)
+                    raise RuntimeError(f"Block used size {used_size} is not equal to the data size {data_size}")
+                fd.write_array(data)
 
     @property
     def data(self):
@@ -1219,6 +1296,7 @@ class UnloadedBlock:
         self._array_storage = 'internal'
         self._input_compression = None
         self._output_compression = 'input'
+        self._output_compression_kwargs = {}
         self._checksum = None
         self._should_memmap = memmap
         self._memmapped = False
