@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import importlib
 
 from .exploded import implode, explode
@@ -10,10 +7,11 @@ from .diff import diff
 from .tags import list_tags
 from .extension import find_extensions
 from .info import info
+from .edit import edit
 
 
 __all__ = ['implode', 'explode', 'to_yaml', 'defragment', 'diff', 'list_tags',
-    'find_extensions', 'info']
+    'find_extensions', 'info', 'edit']
 
 
 # Extracting ASDF-in-FITS files requires Astropy
