@@ -20,13 +20,13 @@ def test_info_command(capsys):
 
     assert main.main_from_args(["info", file_path]) == 0
     captured = capsys.readouterr()
-    assert "root.tree" in captured.out
+    assert "root" in captured.out
     assert "frames" in captured.out
     original_len = len(captured.out.split("\n"))
 
     assert main.main_from_args(["info", "--max-rows", str(original_len - 5), file_path]) == 0
     captured = capsys.readouterr()
-    assert "root.tree" in captured.out
+    assert "root" in captured.out
     assert "frames" in captured.out
     new_len = len(captured.out.split("\n"))
     assert new_len < original_len
