@@ -1,12 +1,10 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
-
 import numpy as np
 
 
 ASDF_MAGIC = b'#ASDF'
 BLOCK_MAGIC = b'\xd3BLK'
+FITS_MAGIC = b'SIMPLE'
+
 BLOCK_HEADER_BOILERPLATE_SIZE = 6
 
 ASDF_STANDARD_COMMENT = b'ASDF_STANDARD'
@@ -26,3 +24,9 @@ STSCI_SCHEMA_TAG_BASE = 'tag:stsci.edu:asdf'
 
 
 BLOCK_FLAG_STREAMED = 0x1
+
+
+# ASDF max number size
+MAX_BITS = 63
+MAX_NUMBER = (1<<MAX_BITS) - 1
+MIN_NUMBER = -((1<<MAX_BITS) - 2)
