@@ -1,7 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
-
 class AsdfWarning(Warning):
     """
     The base warning class from which all ASDF warnings should inherit.
@@ -16,3 +12,9 @@ class AsdfConversionWarning(AsdfWarning):
     """
     Warning class used for failures to convert data into custom types.
     """
+
+class DelimiterNotFoundError(ValueError):
+    """
+    Indicates that a delimiter was not found when reading or
+    seeking through a file.
+    """
