@@ -1,7 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
-
 """
 Utilities for embedded ADSF files in FITS.
 """
@@ -50,13 +46,6 @@ class _FitsBlock:
     def array_storage(self):
         return 'fits'
 
-    def override_byteorder(self, byteorder):
-        # FITS data is always stored in big-endian byte order.
-        # The data array may not report big-endian, but we want
-        # the value written to the tree to match the actual
-        # byte order on disk.
-        return 'big'
-
     @property
     def trust_data_dtype(self):
         # astropy.io.fits returns arrays in native byte order
@@ -134,11 +123,15 @@ class AsdfInFits(asdf.AsdfFile):
     Create a FITS file with ASDF structure, based on an existing FITS
     file::
 
+        import numpy as np
         from astropy.io import fits
 
-        hdulist = fits.HDUList()
-        hdulist.append(fits.ImageHDU(np.arange(512, dtype=float), name='SCI'))
-        hdulist.append(fits.ImageHDU(np.arange(512, dtype=float), name='DQ'))
+        from asdf import fits_embed
+
+        hdulist = fits.HDUList([
+            fits.PrimaryHDU(),
+            fits.ImageHDU(np.arange(512, dtype=np.float32), name='SCI'),
+            fits.ImageHDU(np.zeros(512, dtype=np.int16), name='DQ')])
 
         tree = {
             'model': {
@@ -181,7 +174,7 @@ class AsdfInFits(asdf.AsdfFile):
     def open(cls, fd, uri=None, validate_checksums=False, extensions=None,
              ignore_version_mismatch=True, ignore_unrecognized_tag=False,
              strict_extension_check=False, ignore_missing_extensions=False,
-             validate_on_read=True):
+             **kwargs):
         """Creates a new AsdfInFits object based on given input data
 
         Parameters
@@ -200,10 +193,11 @@ class AsdfInFits(asdf.AsdfFile):
             If `True`, validate the blocks against their checksums.
             Requires reading the entire file, so disabled by default.
 
-        extensions : list of AsdfExtension, optional
-            A list of extensions to the ASDF to support when reading
-            and writing ASDF files.  See `asdf.types.AsdfExtension` for
-            more information.
+        extensions : object, optional
+            Additional extensions to use when reading and writing the file.
+            May be any of the following: `asdf.extension.AsdfExtension`,
+            `asdf.extension.Extension`, `asdf.extension.AsdfExtensionList`
+            or a `list` extensions.
 
         ignore_version_mismatch : bool, optional
             When `True`, do not raise warnings for mismatched schema versions.
@@ -221,8 +215,9 @@ class AsdfInFits(asdf.AsdfFile):
             to `False`.
 
         validate_on_read : bool, optional
-            When `True`, validate the newly opened file against tag and custom
-            schemas.  Recommended unless the file is already known to be valid.
+            DEPRECATED. When `True`, validate the newly opened file against tag
+            and custom schemas.  Recommended unless the file is already known
+            to be valid.
         """
         return cls._open_impl(fd, uri=uri,
                        validate_checksums=validate_checksums,
@@ -231,13 +226,13 @@ class AsdfInFits(asdf.AsdfFile):
                        ignore_unrecognized_tag=ignore_unrecognized_tag,
                        strict_extension_check=strict_extension_check,
                        ignore_missing_extensions=ignore_missing_extensions,
-                       validate_on_read=validate_on_read)
+                       **kwargs)
 
     @classmethod
     def _open_impl(cls, fd, uri=None, validate_checksums=False, extensions=None,
              ignore_version_mismatch=True, ignore_unrecognized_tag=False,
-             strict_extension_check=False, _extension_metadata=None,
-             ignore_missing_extensions=False, validate_on_read=True):
+             strict_extension_check=False,
+             ignore_missing_extensions=False, **kwargs):
 
         close_hdulist = False
         if isinstance(fd, fits.hdu.hdulist.HDUList):
@@ -253,11 +248,9 @@ class AsdfInFits(asdf.AsdfFile):
                 msg = "Failed to parse given file '{}'. Is it FITS?"
                 raise ValueError(msg.format(uri))
 
-        self = cls(hdulist, uri=uri, extensions=extensions,
+        self = cls(hdulist, uri=uri,
                    ignore_version_mismatch=ignore_version_mismatch,
                    ignore_unrecognized_tag=ignore_unrecognized_tag)
-        if _extension_metadata is not None:
-            self._extension_metadata = _extension_metadata
 
         self._close_hdulist = close_hdulist
 
@@ -267,14 +260,15 @@ class AsdfInFits(asdf.AsdfFile):
             # This means there is no ASDF extension
             return self
 
-        buff = io.BytesIO(asdf_extension.data)
+        generic_file = generic_io.get_file(io.BytesIO(asdf_extension.data), mode='r', uri=uri)
 
         try:
-            return cls._open_asdf(self, buff, uri=uri, mode='r',
+            return cls._open_asdf(self, generic_file,
                               validate_checksums=validate_checksums,
+                              extensions=extensions,
                               strict_extension_check=strict_extension_check,
                               ignore_missing_extensions=ignore_missing_extensions,
-                              validate_on_read=validate_on_read)
+                              **kwargs)
         except RuntimeError:
             self.close()
             raise
@@ -291,8 +285,8 @@ class AsdfInFits(asdf.AsdfFile):
             return fits.BinTableHDU.from_columns([column], name=ASDF_EXTENSION_NAME)
 
     def _update_asdf_extension(self, all_array_storage=None,
-                               all_array_compression=None, auto_inline=None,
-                               pad_blocks=False, use_image_hdu=False):
+                               all_array_compression=None,
+                               pad_blocks=False, use_image_hdu=False, **kwargs):
         if self.blocks.streamed_block is not None:
             raise ValueError(
                 "Can not save streamed data to ASDF-in-FITS file.")
@@ -301,30 +295,35 @@ class AsdfInFits(asdf.AsdfFile):
         super(AsdfInFits, self).write_to(
             buff, all_array_storage=all_array_storage,
             all_array_compression=all_array_compression,
-            auto_inline=auto_inline, pad_blocks=pad_blocks,
-            include_block_index=False)
+            pad_blocks=pad_blocks,
+            include_block_index=False, **kwargs)
 
         if ASDF_EXTENSION_NAME in self._hdulist:
             del self._hdulist[ASDF_EXTENSION_NAME]
         self._hdulist.append(self._create_hdu(buff, use_image_hdu))
 
     def write_to(self, filename, all_array_storage=None,
-                 all_array_compression=None, auto_inline=None,
+                 all_array_compression=None,
                  pad_blocks=False, use_image_hdu=False, *args, **kwargs):
+        if "auto_inline" in kwargs:
+            asdf_kwargs = {"auto_inline": kwargs.pop("auto_inline")}
+        else:
+            asdf_kwargs = {}
+
         self._update_asdf_extension(
             all_array_storage=all_array_storage,
             all_array_compression=all_array_compression,
-            auto_inline=auto_inline, pad_blocks=pad_blocks,
-            use_image_hdu=use_image_hdu)
+            pad_blocks=pad_blocks,
+            use_image_hdu=use_image_hdu, **asdf_kwargs)
 
         self._hdulist.writeto(filename, *args, **kwargs)
 
     def update(self, all_array_storage=None, all_array_compression=None,
-               auto_inline=None, pad_blocks=False):
+               pad_blocks=False, **kwargs):
         raise NotImplementedError(
             "In-place update is not currently implemented for ASDF-in-FITS")
 
         self._update_asdf_extension(
             all_array_storage=all_array_storage,
             all_array_compression=all_array_compression,
-            auto_inline=auto_inline, pad_blocks=pad_blocks)
+            pad_blocks=pad_blocks, **kwargs)
