@@ -1,5 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
 """
 This provides abstractions around a number of different file and
 stream types available to Python so that they are always used in the
@@ -13,22 +11,19 @@ import io
 import os
 import re
 import sys
-import math
 import pathlib
 import tempfile
-import platform
-from distutils.version import LooseVersion
 
 from os import SEEK_SET, SEEK_CUR, SEEK_END
 
-import http.client
-from urllib import parse as urlparse
-from urllib.request import url2pathname
+from urllib.request import url2pathname, urlopen
 
 import numpy as np
 
 from . import util
+from .exceptions import DelimiterNotFoundError
 from .extern import atomicfile
+from .util import patched_urllib_parse
 
 
 __all__ = ['get_file', 'get_uri', 'resolve_uri', 'relative_uri']
@@ -65,89 +60,14 @@ def _check_bytes(fd, mode):
     return True
 
 
-if (sys.platform == 'darwin' and
-    LooseVersion(platform.mac_ver()[0]) < LooseVersion('10.9')):  # pragma: no cover
-    def _array_fromfile(fd, size):
-        chunk_size = 1024 ** 3
-        if size < chunk_size:
-            return np.fromfile(fd, dtype=np.uint8, count=size)
-        else:
-            array = np.empty(size, dtype=np.uint8)
-            for beg in range(0, size, chunk_size):
-                end = min(size, beg + chunk_size)
-                array[beg:end] = np.fromfile(fd, dtype=np.uint8, count=end - beg)
-            return array
-else:
-    def _array_fromfile(fd, size):
-        return np.fromfile(fd, dtype=np.uint8, count=size)
-
-
-_array_fromfile.__doc__ = """
-Load a binary array from a real file object.
-
-Parameters
-----------
-fd : real file object
-
-size : integer
-    Number of bytes to read.
-"""
-
-
-def _array_tofile_chunked(write, array, chunksize):  # pragma: no cover
-    array = array.view(np.uint8)
-    for i in range(0, array.nbytes, chunksize):
-        write(array[i:i + chunksize].data)
-
-
-def _array_tofile_simple(fd, write, array):
-    return write(array.data)
-
-
-if sys.platform == 'darwin':  # pragma: no cover
-    def _array_tofile(fd, write, array):
-        # This value is currently set as a workaround for a known bug in Python
-        # on OSX. Individual writes must be less than 2GB, which necessitates
-        # the chunk size here if we want it to remain a power of 2.
-        # See https://bugs.python.org/issue24658.
-        OSX_WRITE_LIMIT = 2 ** 30
-        if fd is None or array.nbytes >= OSX_WRITE_LIMIT and array.nbytes % 4096 == 0:
-            return _array_tofile_chunked(write, array, OSX_WRITE_LIMIT)
-        return _array_tofile_simple(fd, write, array)
-elif sys.platform.startswith('win'):  # pragma: no cover
-    def _array_tofile(fd, write, array):
-        WIN_WRITE_LIMIT = 2 ** 30
-        return _array_tofile_chunked(write, array, WIN_WRITE_LIMIT)
-else:
-    _array_tofile = _array_tofile_simple
-
-
-_array_tofile.__doc__ = """
-Write an array to a file.
-
-Parameters
-----------
-fd : real file object
-   If fd is provided, must be a real system file as supported by
-   numpy.tofile.  May be None, in which case all writing will be done
-   through the `write` method.
-
-write : callable
-   A callable that writes bytes to the file.
-
-array : Numpy array
-   Must be an underlying data array, not a view.
-"""
-
-
 def resolve_uri(base, uri):
     """
     Resolve a URI against a base URI.
     """
     if base is None:
         base = ''
-    resolved = urlparse.urljoin(base, uri)
-    parsed = urlparse.urlparse(resolved)
+    resolved = patched_urllib_parse.urljoin(base, uri)
+    parsed = patched_urllib_parse.urlparse(resolved)
     if parsed.path != '' and not parsed.path.startswith('/'):
         raise ValueError(
             "Resolved to relative URL")
@@ -158,8 +78,8 @@ def relative_uri(source, target):
     """
     Make a relative URI from source to target.
     """
-    su = urlparse.urlparse(source)
-    tu = urlparse.urlparse(target)
+    su = patched_urllib_parse.urlparse(source)
+    tu = patched_urllib_parse.urlparse(target)
     extra = list(tu[3:])
     relative = None
     if tu[0] == '' and tu[1] == '':
@@ -177,7 +97,7 @@ def relative_uri(source, target):
             relative = os.path.relpath(tu[2], os.path.dirname(su[2]))
     if relative == '.':
         relative = ''
-    relative = urlparse.urlunparse(["", "", relative] + extra)
+    relative = patched_urllib_parse.urlunparse(["", "", relative] + extra)
     return relative
 
 
@@ -212,18 +132,18 @@ class _TruncatedReader:
             return content
 
         if nbytes is None:
-            content = self._fd._peek()
+            content = self._fd.peek()
         elif nbytes <= len(self._initial_content):
             content = self._initial_content[:nbytes]
             self._initial_content = self._initial_content[nbytes:]
             return content
         else:
-            content = self._fd._peek(nbytes - len(self._initial_content) +
+            content = self._fd.peek(nbytes - len(self._initial_content) +
                                      self._readahead_bytes)
 
         if content == b'':
             if self._exception:
-                raise ValueError("{0} not found".format(self._delimiter_name))
+                raise DelimiterNotFoundError("{0} not found".format(self._delimiter_name))
             self._past_end = True
             return content
 
@@ -237,7 +157,7 @@ class _TruncatedReader:
             self._past_end = True
         elif nbytes is None and self._exception:
             # Read the whole file and didn't find the delimiter
-            raise ValueError("{0} not found".format(self._delimiter_name))
+            raise DelimiterNotFoundError("{0} not found".format(self._delimiter_name))
         else:
             if nbytes:
                 content = content[:nbytes - len(self._initial_content)]
@@ -292,13 +212,18 @@ class GenericFile(metaclass=util.InheritDocstrings):
             raise ValueError(
                 "File-like object must be opened in binary mode.")
 
+        # can't import at the top level due to circular import
+        from .config import get_config
+        self._asdf_get_config = get_config
+
         self._fd = fd
         self._mode = mode
         self._close = close
-        self._blksize = io.DEFAULT_BUFFER_SIZE
         self._size = None
         self._uri = uri
 
+        self.block_size = get_config().io_block_size
+
     def __enter__(self):
         return self
 
@@ -313,6 +238,19 @@ class GenericFile(metaclass=util.InheritDocstrings):
     def block_size(self):
         return self._blksize
 
+    @block_size.setter
+    def block_size(self, block_size):
+        if block_size == -1:
+            try:
+                block_size = os.fstat(self._fd.fileno()).st_blksize
+            except Exception:
+                block_size = io.DEFAULT_BUFFER_SIZE
+
+        if block_size <= 0:
+            raise ValueError(f'block_size ({block_size}) must be > 0')
+
+        self._blksize = block_size
+
     @property
     def mode(self):
         """
@@ -372,7 +310,40 @@ class GenericFile(metaclass=util.InheritDocstrings):
     """
 
     def write_array(self, array):
-        _array_tofile(None, self.write, array.ravel(order='K'))
+        """
+        Write array content to the file.  Array must be 1D contiguous
+        so that this method can avoid making assumptions about the
+        intended memory layout.  Endianness is preserved.
+
+        Parameters
+        ----------
+        array : np.ndarray
+            Must be 1D contiguous.
+        """
+        if len(array.shape) != 1 or not array.flags.contiguous:
+            raise ValueError("Requires 1D contiguous array.")
+
+        self.write(array.data)
+
+    def peek(self, size=-1):
+        """
+        Read bytes of the file without consuming them.  This method
+        must be implemented by all GenericFile implementations that
+        provide ASDF input (those that aren't seekable should use a
+        buffer to store peeked bytes).
+
+        Parameters
+        ----------
+        size : int
+            Number of bytes to peek, or -1 to peek all remaining bytes.
+        """
+        if self.seekable():
+            cursor = self.tell()
+            content = self.read(size)
+            self.seek(cursor, SEEK_SET)
+            return content
+        else:
+            raise RuntimeError("Non-seekable file")
 
     def seek(self, offset, whence=0):
         """
@@ -492,7 +463,7 @@ class GenericFile(metaclass=util.InheritDocstrings):
 
         Raises
         ------
-        ValueError :
+        DelimiterNotFoundError :
             If the delimiter is not found before the end of the file.
         """
         buff = io.BytesIO()
@@ -541,7 +512,7 @@ class GenericFile(metaclass=util.InheritDocstrings):
 
         Raises
         ------
-        ValueError :
+        DelimiterNotFoundError :
             If the delimiter is not found before the end of the file.
         """
         raise NotImplementedError()
@@ -580,27 +551,28 @@ class GenericFile(metaclass=util.InheritDocstrings):
 
         Returns
         -------
-        content : bytes
-            The content from the current position in the file, up to
-            the delimiter.  Includes the delimiter if `include` is
-            ``True``.
+        bool
+            ``True`` if the delimiter was found.
 
         Raises
         ------
-        ValueError :
-            If the delimiter is not found before the end of the file.
+        DelimiterNotFoundError :
+            If ``exception`` is enabled and the delimiter is not found
+            before the end of the file.
         """
         reader = self.reader_until(
             delimiter, readahead_bytes, delimiter_name=delimiter_name,
             include=include, initial_content=initial_content,
-            exception=exception)
-        while True:
-            try:
-                content = reader.read(self.block_size)
-            except ValueError:
+            exception=True)
+        try:
+            while reader.read(self.block_size) != b'':
+                pass
+            return True
+        except DelimiterNotFoundError as e:
+            if exception:
+                raise e
+            else:
                 return False
-            if content == b'':
-                return True
 
     def fast_forward(self, size):
         """
@@ -677,12 +649,6 @@ class RandomAccessFile(GenericFile):
     def seekable(self):
         return True
 
-    def _peek(self, size=-1):
-        cursor = self.tell()
-        content = self.read(size)
-        self.seek(cursor, SEEK_SET)
-        return content
-
     def reader_until(self, delimiter, readahead_bytes, delimiter_name=None,
                      include=True, initial_content=b'', exception=True):
         return _TruncatedReader(
@@ -734,13 +700,8 @@ class RealFile(RandomAccessFile):
     """
     def __init__(self, fd, mode, close=False, uri=None):
         super(RealFile, self).__init__(fd, mode, close=close, uri=uri)
+
         stat = os.fstat(fd.fileno())
-        if sys.platform.startswith('win'):  # pragma: no cover
-            # There appears to be reliable way to get block size on Windows,
-            # so just choose a reasonable default
-            self._blksize = io.DEFAULT_BUFFER_SIZE
-        else:
-            self._blksize = stat.st_blksize
         self._size = stat.st_size
         if (uri is None and
             isinstance(fd.name, str)):
@@ -751,7 +712,10 @@ class RealFile(RandomAccessFile):
             arr.flush()
             self.fast_forward(len(arr.data))
         else:
-            _array_tofile(self._fd, self._fd.write, arr.ravel(order='K'))
+            if len(arr.shape) != 1 or not arr.flags.contiguous:
+                raise ValueError("Requires 1D contiguous array.")
+
+            self._fd.write(arr.data)
 
     def can_memmap(self):
         return True
@@ -767,7 +731,7 @@ class RealFile(RandomAccessFile):
         return mmap
 
     def read_into_array(self, size):
-        return _array_fromfile(self._fd, size)
+        return np.fromfile(self._fd, dtype=np.uint8, count=size)
 
 
 class MemoryIO(RandomAccessFile):
@@ -801,7 +765,7 @@ class InputStream(GenericFile):
         self._fd = fd
         self._buffer = b''
 
-    def _peek(self, size=-1):
+    def peek(self, size=-1):
         if size < 0:
             self._buffer += self._fd.read()
         else:
@@ -877,211 +841,51 @@ class OutputStream(GenericFile):
         self.clear(size)
 
 
-class HTTPConnection(RandomAccessFile):
+def _http_to_temp(init, mode, uri=None):
     """
-    Uses a persistent HTTP connection to request specific ranges of
-    the file and obtain its structure without transferring it in its
-    entirety.
+    Stream the content of an http or https URL to a temporary file.
 
-    It creates a temporary file on the local filesystem and copies
-    blocks into it as needed.  The `_blocks` array is a bitfield that
-    keeps track of which blocks we have.
+    Parameters
+    ----------
+    init : str
+        HTTP or HTTPS URL.
+    mode : str
+        ASDF file mode.  The temporary file will always be opened
+        in w+b mode, but the resulting GenericFile will report itself
+        writable based on this value.
+    uri : str, optional
+        URI against which relative paths within the file are
+        resolved.  If None, the init value will be used.
+
+    Returns
+    -------
+    RealFile
+        Temporary file.
     """
-    # TODO: Handle HTTPS connection
+    from asdf import get_config
 
-    def __init__(self, connection, size, path, uri, first_chunk):
-        self._mode = 'r'
-        self._blksize = io.DEFAULT_BUFFER_SIZE
-        # The underlying HTTPConnection object doesn't track closed
-        # status, so we do that here.
-        self._closed = False
-        self._fd = connection
-        self._path = path
-        self._uri = uri
-
-        # A bitmap of the blocks that we've already read and cached
-        # locally
-        self._blocks = np.zeros(
-            int(math.ceil(size / self._blksize / 8)), np.uint8)
-
-        local_file = tempfile.TemporaryFile()
-        self._local = RealFile(local_file, 'rw', close=True)
-        self._local.truncate(size)
-        self._local.seek(0)
-        self._local.write(first_chunk)
-        self._local.seek(0)
-        self._blocks[0] = 1
-
-        # The size of the entire file
-        self._size = size
-        self._nreads = 0
-
-        # Some methods just short-circuit to the local copy
-        self.seek = self._local.seek
-        self.tell = self._local.tell
-
-    def __exit__(self, type, value, traceback):
-        if not self._closed:
-            self._local.close()
-            if hasattr(self._fd, '__exit__'):
-                self._fd.__exit__(type, value, traceback)
-            else:
-                self._fd.close()
-            self._closed = True
-
-    def close(self):
-        if not self._closed:
-            self._local.close()
-            self._fd.close()
-            self._closed = True
-
-    def is_closed(self):
-        return self._closed
-
-    def _get_range(self, start, end):
-        """
-        Ensure the range of bytes has been copied to the local cache.
-        """
-        if start >= self._size:
-            return
-
-        end = min(end, self._size)
-
-        blocks = self._blocks
-        block_size = self.block_size
-
-        def has_block(x):
-            return blocks[x >> 3] & (1 << (x & 0x7))
-
-        def mark_block(x):
-            blocks[x >> 3] |= (1 << (x & 0x7))
-
-        block_start = start // block_size
-        block_end = end // block_size + 1
-
-        pos = self._local.tell()
+    fd = tempfile.NamedTemporaryFile("w+b")
 
+    block_size = get_config().io_block_size
+    if block_size == -1:
         try:
-            # Between block_start and block_end, some blocks may be
-            # already loaded.  We want to load all of the missing
-            # blocks in as few requests as possible.
-            a = block_start
-            while a < block_end:
-                # Skip over whole groups of blocks at a time
-                while a < block_end and blocks[a >> 3] == 0xff:
-                    a = ((a >> 3) + 1) << 3
-                while a < block_end and has_block(a):
-                    a += 1
-                if a >= block_end:
-                    break
+            block_size = os.fstat(fd.fileno()).st_blksize
+        except Exception:
+            block_size = io.DEFAULT_BUFFER_SIZE
 
-                b = a + 1
-                # Skip over whole groups of blocks at a time
-                while b < block_end and blocks[b >> 3] == 0x0:
-                    b = ((b >> 3) + 1) << 3
-                while b < block_end and not has_block(b):
-                    b += 1
-                if b > block_end:
-                    b = block_end
+    try:
+        # This method is only called with http and https schemes:
+        with urlopen(init) as response: # nosec
+            chunk = response.read(block_size)
+            while len(chunk) > 0:
+                fd.write(chunk)
+                chunk = response.read(block_size)
+        fd.seek(0)
+        return RealFile(fd, mode, close=True, uri=uri or init)
+    except Exception:
+        fd.close()
+        raise
 
-                if a * block_size >= self._size:
-                    return
-
-                headers = {
-                    'Range': 'bytes={0}-{1}'.format(
-                        a * block_size, (b * block_size) - 1)}
-                self._fd.request('GET', self._path, headers=headers)
-                response = self._fd.getresponse()
-                if response.status != 206:
-                    raise IOError("HTTP failed: {0} {1}".format(
-                        response.status, response.reason))
-
-                # Now copy over to the temporary file, block-by-block
-                self._local.seek(a * block_size, os.SEEK_SET)
-                for i in range(a, b):
-                    chunk = response.read(block_size)
-                    self._local.write(chunk)
-                    mark_block(i)
-                response.close()
-
-                self._nreads += 1
-
-                a = b
-        finally:
-            self._local.seek(pos, os.SEEK_SET)
-
-    def read(self, size=-1):
-        if self._closed:
-            raise IOError("read from closed connection")
-
-        pos = self._local.tell()
-
-        # Adjust size so it doesn't go beyond the end of the file
-        if size < 0 or pos + size > self._size:
-            size = self._size - pos
-
-        # On Python 3, reading 0 bytes from a socket causes it to stop
-        # working, so avoid doing that at all costs.
-        if size == 0:
-            return b''
-
-        self._get_range(pos, pos + size)
-        return self._local.read(size)
-
-    def read_into_array(self, size):
-        if self._closed:
-            raise IOError("read from closed connection")
-
-        pos = self._local.tell()
-
-        if pos + size > self._size:
-            raise IOError("Read past end of file.")
-
-        self._get_range(pos, pos + size)
-        return self._local.memmap_array(pos, size)
-
-
-def _make_http_connection(init, mode, uri=None):
-    """
-    Creates a HTTPConnection instance if the HTTP server supports
-    Range requests, otherwise falls back to a generic InputStream.
-    """
-    parsed = urlparse.urlparse(init)
-    connection = http.client.HTTPConnection(parsed.netloc)
-    connection.connect()
-
-    block_size = io.DEFAULT_BUFFER_SIZE
-
-    # We request a range of the whole file ("0-") to check if the
-    # server understands that header entry, and also to get the
-    # size of the entire file
-    headers = {'Range': 'bytes=0-'}
-    connection.request('GET', parsed.path, headers=headers)
-    response = connection.getresponse()
-    if response.status // 100 != 2:
-        raise IOError("HTTP failed: {0} {1}".format(
-            response.status, response.reason))
-
-    # Status 206 means a range was returned.  If it's anything else
-    # that indicates the server probably doesn't support Range
-    # headers.
-    if (response.status != 206 or
-        response.getheader('accept-ranges', None) != 'bytes' or
-        response.getheader('content-range', None) is None or
-        response.getheader('content-length', None) is None):
-        # Fall back to a regular input stream, but we don't
-        # need to open a new connection.
-        response.close = connection.close
-        return InputStream(response, mode, uri=uri or init, close=True)
-
-    # Since we'll be requesting chunks, we can't read at all with the
-    # current request (because we can't abort it), so just close and
-    # start over
-    size = int(response.getheader('content-length'))
-    first_chunk = response.read(block_size)
-    response.close()
-    return HTTPConnection(connection, size, parsed.path, uri or init,
-                          first_chunk)
 
 def get_uri(file_obj):
     """
@@ -1168,12 +972,12 @@ def get_file(init, mode='r', uri=None, close=False):
         return GenericWrapper(init)
 
     elif isinstance(init, (str, pathlib.Path)):
-        parsed = urlparse.urlparse(str(init))
+        parsed = patched_urllib_parse.urlparse(str(init))
         if parsed.scheme in ['http', 'https']:
             if 'w' in mode:
                 raise ValueError(
                     "HTTP connections can not be opened for writing")
-            return _make_http_connection(init, mode, uri=uri)
+            return _http_to_temp(init, mode, uri=uri)
         elif parsed.scheme in _local_file_schemes:
             if mode == 'rw':
                 realmode = 'r+b'
