@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 """
 Manages external references in the YAML tree using the `JSON Reference
 standard <http://tools.ietf.org/html/draft-pbryan-zyp-json-ref-03>`__
@@ -13,13 +10,14 @@ import weakref
 
 import numpy as np
 
-from urllib import parse as urlparse
-
 from .types import AsdfType
 from . import generic_io
 from . import treeutil
 from . import util
 
+from .util import patched_urllib_parse
+
+
 __all__ = [
     'resolve_fragment', 'Reference', 'find_references', 'resolve_references',
     'make_reference']
@@ -30,7 +28,7 @@ def resolve_fragment(tree, pointer):
     Resolve a JSON Pointer within the tree.
     """
     pointer = pointer.lstrip(u"/")
-    parts = urlparse.unquote(pointer).split(u"/") if pointer else []
+    parts = patched_urllib_parse.unquote(pointer).split(u"/") if pointer else []
 
     for part in parts:
         part = part.replace(u"~1", u"/").replace(u"~0", u"~")
@@ -60,15 +58,14 @@ class Reference(AsdfType):
         self._base_uri = base_uri
         self._target = target
 
-    def _get_target(self, do_not_fill_defaults=False):
+    def _get_target(self, **kwargs):
         if self._target is None:
             base_uri = self._base_uri
             if base_uri is None:
                 base_uri = self._asdffile().uri
             uri = generic_io.resolve_uri(base_uri, self._uri)
-            asdffile = self._asdffile().open_external(
-                uri, do_not_fill_defaults=do_not_fill_defaults)
-            parts = urlparse.urlparse(self._uri)
+            asdffile = self._asdffile().open_external(uri, **kwargs)
+            parts = patched_urllib_parse.urlparse(self._uri)
             fragment = parts.fragment
             self._target = resolve_fragment(asdffile.tree, fragment)
         return self._target
@@ -109,8 +106,8 @@ class Reference(AsdfType):
     def __array__(self):
         return np.asarray(self._get_target())
 
-    def __call__(self, do_not_fill_defaults=False):
-        return self._get_target(do_not_fill_defaults=do_not_fill_defaults)
+    def __call__(self, **kwargs):
+        return self._get_target(**kwargs)
 
     def __contains__(self, item):
         return item in self._get_target()
@@ -142,14 +139,14 @@ def find_references(tree, ctx):
         tree, do_find, ignore_implicit_conversion=ctx._ignore_implicit_conversion)
 
 
-def resolve_references(tree, ctx, do_not_fill_defaults=False):
+def resolve_references(tree, ctx, **kwargs):
     """
     Resolve all of the references in the tree, by loading the external
     data and inserting it directly into the tree.
     """
     def do_resolve(tree):
         if isinstance(tree, Reference):
-            return tree(do_not_fill_defaults=do_not_fill_defaults)
+            return tree(**kwargs)
         return tree
 
     tree = find_references(tree, ctx)
