@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import json
 import datetime
 import warnings
@@ -8,7 +5,7 @@ import copy
 from numbers import Integral
 from functools import lru_cache
 from collections import OrderedDict
-from urllib import parse as urlparse
+from collections.abc import Mapping
 
 from jsonschema import validators as mvalidators
 from jsonschema.exceptions import ValidationError
@@ -16,6 +13,7 @@ from jsonschema.exceptions import ValidationError
 import yaml
 import numpy as np
 
+from .config import get_config
 from . import constants
 from . import generic_io
 from . import reference
@@ -23,8 +21,12 @@ from . import treeutil
 from . import util
 from . import extension
 from . import yamlutil
+from . import versioning
+from . import tagged
 from .exceptions import AsdfDeprecationWarning, AsdfWarning
 
+from .util import patched_urllib_parse
+
 
 YAML_SCHEMA_METASCHEMA_ID = 'http://stsci.edu/schemas/yaml-schema/draft-01'
 
@@ -77,18 +79,29 @@ def _type_to_tag(type_):
     return None
 
 
-def validate_tag(validator, tagname, instance, schema):
-
+def validate_tag(validator, tag_pattern, instance, schema):
+    """
+    Implements the tag validation directive, which checks the
+    tag against a pattern that may include wildcards.  See
+    `asdf.util.uri_match` for details on the matching behavior.
+    """
     if hasattr(instance, '_tag'):
         instance_tag = instance._tag
     else:
         # Try tags for known Python builtins
         instance_tag = _type_to_tag(type(instance))
 
-    if instance_tag is not None and instance_tag != tagname:
+    if instance_tag is None:
+        yield ValidationError(
+            "mismatched tags, wanted '{}', got unhandled object type '{}'".format(
+                tag_pattern, util.get_class_name(instance)
+            )
+        )
+
+    if not util.uri_match(tag_pattern, instance_tag):
         yield ValidationError(
             "mismatched tags, wanted '{0}', got '{1}'".format(
-                tagname, instance_tag))
+                tag_pattern, instance_tag))
 
 
 def validate_propertyOrder(validator, order, instance, schema):
@@ -293,19 +306,28 @@ def _create_validator(validators=YAML_VALIDATORS, visit_repeat_nodes=False):
                 if _schema is None:
                     tag = getattr(instance, '_tag', None)
                     if tag is not None:
-                        schema_path = self.ctx.resolver(tag)
-                        if schema_path != tag:
+                        if self.serialization_context.extension_manager.handles_tag(tag):
+                            tag_def = self.serialization_context.extension_manager.get_tag_definition(tag)
+                            schema_uris = tag_def.schema_uris
+                        else:
+                            schema_uris = [self.ctx.tag_mapping(tag)]
+                            if schema_uris[0] == tag:
+                                schema_uris = []
+
+                        # Must validate against all schema_uris
+                        for schema_uri in schema_uris:
                             try:
-                                s = _load_schema_cached(schema_path, self.ctx.resolver, False, False)
+                                s = _load_schema_cached(schema_uri, self.ctx.resolver, False, False)
                             except FileNotFoundError:
                                 msg = "Unable to locate schema file for '{}': '{}'"
-                                warnings.warn(msg.format(tag, schema_path), AsdfWarning)
+                                warnings.warn(msg.format(tag, schema_uri), AsdfWarning)
                                 s = {}
                             if s:
-                                with self.resolver.in_scope(schema_path):
+                                with self.resolver.in_scope(schema_uri):
                                     for x in super(ASDFValidator, self).iter_errors(instance, s):
                                         yield x
 
+
                     if isinstance(instance, dict):
                         for val in instance.values():
                             for x in self.iter_errors(val):
@@ -337,6 +359,19 @@ def _load_schema(url):
 
 def _make_schema_loader(resolver):
     def load_schema(url):
+        # Check if this is a URI provided by the new
+        # Mapping API:
+        resource_manager = get_config().resource_manager
+        if url in resource_manager:
+            content = resource_manager[url]
+            # The jsonschema metaschemas are JSON, but pyyaml
+            # doesn't mind.
+            # The following call to yaml.load is safe because we're
+            # using a loader that inherits from pyyaml's SafeLoader.
+            result = yaml.load(content, Loader=yamlutil.AsdfLoader) # nosec
+            return result, url
+
+        # If not, fall back to fetching the schema the old way:
         url = resolver(str(url))
         return _load_schema(url)
     return load_schema
@@ -349,27 +384,24 @@ def _make_resolver(url_mapping):
     def get_schema(url):
         return schema_loader(url)[0]
 
-    for x in ['http', 'https', 'file', 'tag']:
+    for x in ['http', 'https', 'file', 'tag', 'asdf']:
         handlers[x] = get_schema
 
+    # Supplying our own implementation of urljoin_cache
+    # allows asdf:// URIs to be resolved correctly.
+    urljoin_cache = lru_cache(1024)(patched_urllib_parse.urljoin)
+
     # We set cache_remote=False here because we do the caching of
     # remote schemas here in `load_schema`, so we don't need
     # jsonschema to do it on our behalf.  Setting it to `True`
     # counterintuitively makes things slower.
     return mvalidators.RefResolver(
-        '', {}, cache_remote=False, handlers=handlers)
-
-
-def _load_draft4_metaschema():
-    from jsonschema import _utils
-    return _utils.load_schema('draft4')
-
-
-# This is a list of schema that we have locally on disk but require
-# special methods to obtain
-HARDCODED_SCHEMA = {
-    'http://json-schema.org/draft-04/schema': _load_draft4_metaschema
-}
+        '',
+        {},
+        cache_remote=False,
+        handlers=handlers,
+        urljoin_cache=urljoin_cache,
+    )
 
 
 @lru_cache()
@@ -427,51 +459,73 @@ def load_schema(url, resolver=None, resolve_references=False,
     )
 
 
+def _safe_resolve(resolver, json_id, uri):
+    """
+    This function handles the tricky task of resolving a schema URI
+    in the presence of both new and legacy extensions.
+
+    There are two senses of "resolve" here: one is to resolve the URI
+    to a file:// URL using the legacy extension resolver object. The other
+    is to resolve relative URIs against the id of the current schema document,
+    which is what generic_io.resolve_uri does.
+
+    For URIs associated with new-style extensions, we want to resolve with
+    generic_io.resolve_uri, but not with the resolver object, otherwise we risk
+    mangling URIs that share a prefix with a resolver mapping.
+    """
+    # We can't use urllib.parse here because tag: URIs don't
+    # parse correctly.
+    parts = uri.split("#")
+    base = parts[0]
+    if len(parts) > 1:
+        fragment = parts[1]
+    else:
+        fragment = ""
+
+    # The generic_io.resolve_uri method cannot operate on tag: URIs.
+    # New-style extensions don't support $ref with a tag URI target anyway,
+    # so it's safe to feed this through the resolver right away.
+    if base.startswith("tag:"):
+        base = resolver(base)
+
+    # Resolve relative URIs (e.g., #foo/bar, ../foo/bar) against
+    # the current schema id.
+    base = generic_io.resolve_uri(json_id, base)
+
+    # Use the resolver object only if the URI does not belong to one
+    # of the new-style extensions.
+    if base not in get_config().resource_manager:
+        base = resolver(base)
+
+    return base, fragment
+
+
 @lru_cache()
 def _load_schema_cached(url, resolver, resolve_references, resolve_local_refs):
     loader = _make_schema_loader(resolver)
-    if url in HARDCODED_SCHEMA:
-        schema = HARDCODED_SCHEMA[url]()
-    else:
-        schema, url = loader(url)
+    schema, url = loader(url)
 
-    # Resolve local references
-    if resolve_local_refs:
-        def resolve_local(node, json_id):
-            if isinstance(node, dict) and '$ref' in node:
-                ref_url = resolver(node['$ref'])
-                if ref_url.startswith('#'):
-                    parts = urlparse.urlparse(ref_url)
-                    subschema_fragment = reference.resolve_fragment(
-                        schema, parts.fragment)
-                    return subschema_fragment
-            return node
-
-        schema = treeutil.walk_and_modify(schema, resolve_local)
-
-    if resolve_references:
+    if resolve_references or resolve_local_refs:
         def resolve_refs(node, json_id):
             if json_id is None:
                 json_id = url
+
             if isinstance(node, dict) and '$ref' in node:
-                suburl = generic_io.resolve_uri(json_id, resolver(node['$ref']))
-                parts = urlparse.urlparse(suburl)
-                fragment = parts.fragment
-                if len(fragment):
-                    suburl_path = suburl[:-(len(fragment) + 1)]
-                else:
-                    suburl_path = suburl
-                suburl_path = resolver(suburl_path)
-                if suburl_path == url:
+                suburl_base, suburl_fragment = _safe_resolve(resolver, json_id, node['$ref'])
+
+                if suburl_base == url or suburl_base == schema.get("id"):
+                    # This is a local ref, which we'll resolve in both cases.
                     subschema = schema
+                elif resolve_references:
+                    # Only resolve non-local refs when the flag is set.
+                    subschema = load_schema(suburl_base, resolver, True)
                 else:
-                    subschema = load_schema(suburl_path, resolver, True)
+                    # Otherwise return the $ref unmodified.
+                    return node
 
-                subschema_fragment = reference.resolve_fragment(
-                    subschema, fragment)
-                return subschema_fragment
-
-            return node
+                return reference.resolve_fragment(subschema, suburl_fragment)
+            else:
+                return node
 
         schema = treeutil.walk_and_modify(schema, resolve_refs)
 
@@ -479,7 +533,8 @@ def _load_schema_cached(url, resolver, resolve_references, resolve_local_refs):
 
 
 def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
-                  *args, _visit_repeat_nodes=False, **kwargs):
+                  *args, _visit_repeat_nodes=False, _serialization_context=None,
+                  **kwargs):
     """
     Get a JSON schema validator object for the given schema.
 
@@ -517,9 +572,12 @@ def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
         from .asdf import AsdfFile
         ctx = AsdfFile()
 
+    if _serialization_context is None:
+        _serialization_context = ctx._create_serialization_context()
+
     if validators is None:
         validators = util.HashableDict(YAML_VALIDATORS.copy())
-        validators.update(ctx._extensions.validators)
+        validators.update(ctx.extension_list.validators)
 
     kwargs['resolver'] = _make_resolver(url_mapping)
 
@@ -531,33 +589,62 @@ def get_validator(schema={}, ctx=None, validators=None, url_mapping=None,
     cls = _create_validator(validators=validators, visit_repeat_nodes=_visit_repeat_nodes)
     validator = cls(schema, *args, **kwargs)
     validator.ctx = ctx
+    validator.serialization_context = _serialization_context
     return validator
 
 
-def validate_large_literals(instance, reading=False):
+def _validate_large_literals(instance, reading):
     """
     Validate that the tree has no large numeric literals.
     """
-    # We can count on 52 bits of precision
-    for instance in treeutil.iter_tree(instance):
+    def _validate(value):
+        if value <= constants.MAX_NUMBER and value >= constants.MIN_NUMBER:
+            return
 
-        if not isinstance(instance, Integral):
-            continue
-
-        if instance <= ((1 << 51) - 1) and instance >= -((1 << 51) - 2):
-            continue
-
-        if not reading:
+        if reading:
+            warnings.warn(
+                f"Invalid integer literal value {value} detected while reading file. "
+                "The value has been read safely, but the file should be "
+                "fixed.",
+                AsdfWarning
+            )
+        else:
             raise ValidationError(
-                "Integer value {0} is too large to safely represent as a "
-                "literal in ASDF".format(instance))
+                f"Integer value {value} is too large to safely represent as a "
+                "literal in ASDF"
+            )
 
-        warnings.warn(
-            "Invalid integer literal value {0} detected while reading file. "
-            "The value has been read safely, but the file should be "
-            "fixed.".format(instance),
-            AsdfWarning
-        )
+    if isinstance(instance, Integral):
+        _validate(instance)
+    elif isinstance(instance, Mapping):
+        for key in instance:
+            if isinstance(key, Integral):
+                _validate(key)
+
+
+def _validate_mapping_keys(instance, reading):
+    """
+    Validate that mappings do not contain illegal key types
+    (as of ASDF Standard 1.6.0, only str, int, and bool are
+    permitted).
+    """
+    if not isinstance(instance, Mapping):
+        return
+
+    for key in instance:
+        if isinstance(key, tagged.Tagged) or not isinstance(key, (str, int, bool)):
+            if reading:
+                warnings.warn(
+                    f"Invalid mapping key {key} detected while reading file. "
+                    "The value has been read safely, but the file should be "
+                    "fixed.",
+                    AsdfWarning
+                )
+            else:
+                raise ValidationError(
+                    f"Mapping key {key} is not permitted.  Valid types: "
+                    "str, int, bool."
+                )
 
 
 def validate(instance, ctx=None, schema={}, validators=None, reading=False,
@@ -598,7 +685,15 @@ def validate(instance, ctx=None, schema={}, validators=None, reading=False,
                               *args, **kwargs)
     validator.validate(instance, _schema=(schema or None))
 
-    validate_large_literals(instance, reading=reading)
+    additional_validators = [_validate_large_literals]
+    if ctx.version >= versioning.RESTRICTED_KEYS_MIN_VERSION:
+        additional_validators.append(_validate_mapping_keys)
+
+    def _callback(instance):
+        for validator in additional_validators:
+            validator(instance, reading)
+
+    treeutil.walk(instance, _callback)
 
 
 def fill_defaults(instance, ctx, reading=False):
@@ -675,7 +770,11 @@ def check_schema(schema, validate_default=True):
 
     resolver = _make_resolver(extension.get_default_resolver())
 
-    cls = mvalidators.create(meta_schema=meta_schema,
-                             validators=validators)
+    cls = mvalidators.create(
+        meta_schema=meta_schema,
+        validators=validators,
+        type_checker=mvalidators.Draft4Validator.TYPE_CHECKER,
+        id_of=mvalidators.Draft4Validator.ID_OF,
+    )
     validator = cls(meta_schema, resolver=resolver)
     validator.validate(schema, _schema=meta_schema)
