@@ -1,5 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
 from ...types import AsdfType
 
 
@@ -47,6 +45,10 @@ class ExtensionMetadata(dict, AsdfType):
     name = 'core/extension_metadata'
     version = '1.0.0'
 
+    @property
+    def extension_uri(self):
+        return self.get('extension_uri')
+
     @property
     def extension_class(self):
         return self['extension_class']
@@ -57,5 +59,12 @@ class ExtensionMetadata(dict, AsdfType):
 
 
 class SubclassMetadata(dict, AsdfType):
+    """
+    The tagged object supported by this class is part of
+    an experimental feature that has since been dropped
+    from this library.  This class remains so that ASDF
+    files that used that feature will still deserialize
+    without warnings.
+    """
     name = 'core/subclass_metadata'
     version = '1.0.0'
