@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import sys
 
 import numpy as np
@@ -9,7 +6,6 @@ from numpy import ma
 from jsonschema import ValidationError
 
 from ...types import AsdfType
-from ... import schema
 from ... import util
 
 
@@ -85,7 +81,10 @@ def asdf_datatype_to_numpy_dtype(datatype, byteorder=None):
     raise ValueError("Unknown datatype {0}".format(datatype))
 
 
-def numpy_byteorder_to_asdf_byteorder(byteorder):
+def numpy_byteorder_to_asdf_byteorder(byteorder, override=None):
+    if override is not None:
+        return override
+
     if byteorder == '=':
         return sys.byteorder
     elif byteorder == '<':
@@ -94,7 +93,7 @@ def numpy_byteorder_to_asdf_byteorder(byteorder):
         return 'big'
 
 
-def numpy_dtype_to_asdf_datatype(dtype, include_byteorder=True):
+def numpy_dtype_to_asdf_datatype(dtype, include_byteorder=True, override_byteorder=None):
     dtype = np.dtype(dtype)
     if dtype.names is not None:
         fields = []
@@ -102,30 +101,30 @@ def numpy_dtype_to_asdf_datatype(dtype, include_byteorder=True):
             field = dtype.fields[name][0]
             d = {}
             d['name'] = name
-            field_dtype, byteorder = numpy_dtype_to_asdf_datatype(field)
+            field_dtype, byteorder = numpy_dtype_to_asdf_datatype(field, override_byteorder=override_byteorder)
             d['datatype'] = field_dtype
             if include_byteorder:
                 d['byteorder'] = byteorder
             if field.shape:
                 d['shape'] = list(field.shape)
             fields.append(d)
-        return fields, numpy_byteorder_to_asdf_byteorder(dtype.byteorder)
+        return fields, numpy_byteorder_to_asdf_byteorder(dtype.byteorder, override=override_byteorder)
 
     elif dtype.subdtype is not None:
-        return numpy_dtype_to_asdf_datatype(dtype.subdtype[0])
+        return numpy_dtype_to_asdf_datatype(dtype.subdtype[0], override_byteorder=override_byteorder)
 
     elif dtype.name in _datatype_names:
-        return dtype.name, numpy_byteorder_to_asdf_byteorder(dtype.byteorder)
+        return dtype.name, numpy_byteorder_to_asdf_byteorder(dtype.byteorder, override=override_byteorder)
 
     elif dtype.name == 'bool':
-        return 'bool8', numpy_byteorder_to_asdf_byteorder(dtype.byteorder)
+        return 'bool8', numpy_byteorder_to_asdf_byteorder(dtype.byteorder, override=override_byteorder)
 
     elif dtype.name.startswith('string') or dtype.name.startswith('bytes'):
         return ['ascii', dtype.itemsize], 'big'
 
     elif dtype.name.startswith('unicode') or dtype.name.startswith('str'):
         return (['ucs4', int(dtype.itemsize / 4)],
-                numpy_byteorder_to_asdf_byteorder(dtype.byteorder))
+                numpy_byteorder_to_asdf_byteorder(dtype.byteorder, override=override_byteorder))
 
     raise ValueError("Unknown dtype {0}".format(dtype))
 
@@ -206,8 +205,6 @@ def numpy_array_to_list(array):
 
     result = ascii_to_unicode(tolist(array))
 
-    schema.validate_large_literals(result)
-
     return result
 
 
@@ -371,6 +368,18 @@ class NDArrayType(AsdfType):
             self._array = None
             raise e from None
 
+    def __getattribute__(self, name):
+        # The presence of these attributes on an NDArrayType instance
+        # can cause problems when the array is passed to other
+        # libraries.
+        # See https://github.com/asdf-format/asdf/issues/1015
+        if name in ("name", "version"):
+            raise AttributeError(
+                f"'{self.__class__.name}' object has no attribute '{name}'"
+            )
+        else:
+            return super().__getattribute__(name)
+
     @classmethod
     def from_tree(cls, node, ctx):
         if isinstance(node, list):
@@ -414,32 +423,68 @@ class NDArrayType(AsdfType):
 
     @classmethod
     def to_tree(cls, data, ctx):
+        # The ndarray-1.0.0 schema does not permit 0 valued strides.
+        # Perhaps we'll want to allow this someday, to efficiently
+        # represent an array of all the same value.
         if any(stride == 0 for stride in data.strides):
             data = np.ascontiguousarray(data)
 
+        # The view computations that follow assume that the base array
+        # is contiguous.  If not, we need to make a copy to avoid
+        # writing a nonsense view.
         base = util.get_array_base(data)
-        shape = data.shape
-        dtype = data.dtype
-        offset = data.ctypes.data - base.ctypes.data
+        if not base.flags.contiguous:
+            data = np.ascontiguousarray(data)
+            base = util.get_array_base(data)
 
-        if data.flags.c_contiguous:
-            strides = None
-        else:
-            strides = data.strides
+        shape = data.shape
 
         block = ctx.blocks.find_or_create_block_for_array(data, ctx)
 
+        if block.array_storage == "fits":
+            # Views over arrays stored in FITS files have some idiosyncracies.
+            # astropy.io.fits always writes arrays C-contiguous with big-endian
+            # byte order, whereas asdf preserves the "contiguousity" and byte order
+            # of the base array.
+            if (block.data.shape != data.shape or
+                block.data.dtype != data.dtype or
+                block.data.ctypes.data != data.ctypes.data or
+                block.data.strides != data.strides):
+                raise ValueError(
+                    "ASDF has only limited support for serializing views over arrays stored "
+                    "in FITS HDUs.  This error likely means that a slice of such an array "
+                    "was found in the ASDF tree.  The slice can be decoupled from the FITS "
+                    "array by calling copy() before assigning it to the tree."
+                )
+
+            offset = 0
+            strides = None
+            dtype, byteorder = numpy_dtype_to_asdf_datatype(
+                data.dtype,
+                include_byteorder=(block.array_storage != "inline"),
+                override_byteorder="big",
+            )
+        else:
+            # Compute the offset relative to the base array and not the
+            # block data, in case the block is compressed.
+            offset = data.ctypes.data - base.ctypes.data
+
+            if data.flags.c_contiguous:
+                strides = None
+            else:
+                strides = data.strides
+
+            dtype, byteorder = numpy_dtype_to_asdf_datatype(
+                data.dtype,
+                include_byteorder=(block.array_storage != "inline"),
+            )
+
         result = {}
 
         result['shape'] = list(shape)
         if block.array_storage == 'streamed':
             result['shape'][0] = '*'
 
-        dtype, byteorder = numpy_dtype_to_asdf_datatype(
-            dtype, include_byteorder=(block.array_storage != 'inline'))
-
-        byteorder = block.override_byteorder(byteorder)
-
         if block.array_storage == 'inline':
             listdata = numpy_array_to_list(data)
             result['data'] = listdata
