@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import io
 import os
 import re
@@ -79,8 +76,9 @@ def test_sharing(tmpdir):
         assert len(list(asdf.blocks.internal_blocks)) == 1
         assert next(asdf.blocks.internal_blocks)._size == 80
 
-        tree['science_data'][0] = 42
-        assert tree['skipping'][0] == 42
+        if 'w' in asdf._mode:
+            tree['science_data'][0] = 42
+            assert tree['skipping'][0] == 42
 
     def check_raw_yaml(content):
         assert b'!core/ndarray' in content
@@ -182,11 +180,12 @@ def test_table_inline(tmpdir):
             'shape': [2]
             }
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, raw_yaml_check_func=check_raw_yaml,
-                                  write_options={'auto_inline': 64})
+    with asdf.config_context() as config:
+        config.array_inline_threshold = 100
+        helpers.assert_roundtrip_tree(tree, tmpdir, raw_yaml_check_func=check_raw_yaml)
 
 
-def test_auto_inline_recursive(tmpdir):
+def test_array_inline_threshold_recursive(tmpdir):
     models = pytest.importorskip('astropy.modeling.models')
 
     aff = models.AffineTransformation2D(matrix=[[1, 2], [3, 4]])
@@ -195,8 +194,9 @@ def test_auto_inline_recursive(tmpdir):
     def check_asdf(asdf):
         assert len(list(asdf.blocks.internal_blocks)) == 0
 
-    helpers.assert_roundtrip_tree(tree, tmpdir, asdf_check_func=check_asdf,
-                                  write_options={'auto_inline': 64})
+    with asdf.config_context() as config:
+        config.array_inline_threshold = 100
+        helpers.assert_roundtrip_tree(tree, tmpdir, asdf_check_func=check_asdf)
 
 
 def test_copy_inline():
@@ -371,7 +371,7 @@ def test_string(tmpdir):
 
 def test_string_table(tmpdir):
     tree = {
-        'table': np.array([(b'foo', 'სამეცნიერო', 42, 53.0)])
+        'table': np.array([(b'foo', 'სამეცნიერო', '42', '53.0')])
         }
 
     helpers.assert_roundtrip_tree(tree, tmpdir)
@@ -476,7 +476,7 @@ def test_masked_array_stay_open_bug(tmpdir):
         with asdf.open(tmppath) as f2:
             np.sum(f2.tree['test'])
 
-    assert len(p.open_files()) == len(orig_open)
+    assert len(p.open_files()) <= len(orig_open)
 
 
 def test_masked_array_repr(tmpdir):
@@ -792,7 +792,7 @@ def test_inline_shape_mismatch():
 @pytest.mark.xfail(
     reason="NDArrays with dtype=object are not currently supported")
 def test_simple_object_array(tmpdir):
-    # See https://github.com/spacetelescope/asdf/issues/383 for feature
+    # See https://github.com/asdf-format/asdf/issues/383 for feature
     # request
     dictdata = np.empty((3, 3), dtype=object)
     for i, _ in enumerate(dictdata.flat):
@@ -804,7 +804,7 @@ def test_simple_object_array(tmpdir):
 @pytest.mark.xfail(
     reason="NDArrays with dtype=object are not currently supported")
 def test_tagged_object_array(tmpdir):
-    # See https://github.com/spacetelescope/asdf/issues/383 for feature
+    # See https://github.com/asdf-format/asdf/issues/383 for feature
     # request
     quantity = pytest.importorskip('astropy.units.quantity')
 
@@ -895,3 +895,26 @@ def test_block_data_change(tmpdir):
         af.update()
         array_after = af.tree["data"].__array__()
         assert array_before is not array_after
+
+
+def test_problematic_class_attributes(tmp_path):
+    """
+    The presence of the "name" and "version" attributes
+    in NDArrayType cause problems when our arrays are used
+    with other libraries.
+
+    See https://github.com/asdf-format/asdf/issues/1015
+    """
+    file_path = tmp_path / "test.asdf"
+    with asdf.AsdfFile() as af:
+        af["arr"] = np.arange(100)
+        af.write_to(file_path)
+
+    with asdf.open(file_path) as af:
+        assert isinstance(af["arr"], ndarray.NDArrayType)
+
+        with pytest.raises(AttributeError):
+            af["arr"].name
+
+        with pytest.raises(AttributeError):
+            af["arr"].version
