@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import os
 import io
 import getpass
@@ -15,12 +13,14 @@ import pytest
 from jsonschema.exceptions import ValidationError
 
 import asdf
+from asdf import get_config, config_context
 from asdf import treeutil
 from asdf import extension
 from asdf import resolver
 from asdf import schema
 from asdf import versioning
 from asdf.exceptions import AsdfDeprecationWarning, AsdfWarning
+from asdf.extension import ExtensionProxy
 from .helpers import (
     assert_tree_match,
     assert_roundtrip_tree,
@@ -29,6 +29,7 @@ from .helpers import (
 )
 
 
+
 def test_get_data_from_closed_file(tmpdir):
     tmpdir = str(tmpdir)
     path = os.path.join(tmpdir, 'test.asdf')
@@ -49,7 +50,7 @@ def test_get_data_from_closed_file(tmpdir):
 def test_no_warning_nan_array(tmpdir):
     """
     Tests for a regression that was introduced by
-    https://github.com/spacetelescope/asdf/pull/557
+    https://github.com/asdf-format/asdf/pull/557
     """
 
     tree = dict(array=np.array([1, 2, np.nan]))
@@ -103,16 +104,37 @@ invalid_software: !core/software-1.0.0
     buff = yaml_to_asdf(content)
 
     with pytest.raises(ValidationError):
-        with asdf.open(buff, validate_on_read=True):
+        get_config().validate_on_read = True
+        with asdf.open(buff):
             pass
 
     buff.seek(0)
 
-    with asdf.open(buff, validate_on_read=False) as af:
+    get_config().validate_on_read = False
+    with asdf.open(buff) as af:
         assert af["invalid_software"]["name"] == "Minesweeper"
         assert af["invalid_software"]["version"] == 3
 
 
+def test_open_stream(tmp_path):
+    file_path = tmp_path / "test.asdf"
+
+    with asdf.AsdfFile() as af:
+        af["foo"] = "bar"
+        af.write_to(file_path)
+
+    class StreamWrapper:
+        def __init__(self, fd):
+            self._fd = fd
+
+        def read(self, size=-1):
+            return self._fd.read(size)
+
+    with file_path.open("rb") as fd:
+        with asdf.open(StreamWrapper(fd)) as af:
+            assert af["foo"] == "bar"
+
+
 def test_atomic_write(tmpdir, small_tree):
     tmpfile = os.path.join(str(tmpdir), 'test.asdf')
 
@@ -125,7 +147,7 @@ def test_atomic_write(tmpdir, small_tree):
 
 def test_overwrite(tmpdir):
     # This is intended to reproduce the following issue:
-    # https://github.com/spacetelescope/asdf/issues/100
+    # https://github.com/asdf-format/asdf/issues/100
     tmpfile = os.path.join(str(tmpdir), 'test.asdf')
     aff = models.AffineTransformation2D(matrix=[[1, 2], [3, 4]])
     f = asdf.AsdfFile()
@@ -139,7 +161,7 @@ def test_overwrite(tmpdir):
 
 
 def test_default_version():
-    # See https://github.com/spacetelescope/asdf/issues/364
+    # See https://github.com/asdf-format/asdf/issues/364
 
     version_map = versioning.get_version_map(versioning.default_version)
 
@@ -207,7 +229,7 @@ def test_walk_and_modify_remove_keys():
 
     def func(x):
         if x == 42:
-            return None
+            return treeutil.RemoveNode
         return x
 
     tree2 = treeutil.walk_and_modify(tree, func)
@@ -216,6 +238,23 @@ def test_walk_and_modify_remove_keys():
     assert 'bar' in tree2
 
 
+def test_walk_and_modify_retain_none():
+    tree = {
+        'foo': 42,
+        'bar': None
+    }
+
+    def func(x):
+        if x == 42:
+            return None
+        return x
+
+    tree2 = treeutil.walk_and_modify(tree, func)
+
+    assert tree2['foo'] is None
+    assert tree2['bar'] is None
+
+
 def test_copy(tmpdir):
     tmpdir = str(tmpdir)
 
@@ -268,7 +307,7 @@ def test_access_tree_outside_handler(tmpdir):
 
 def test_context_handler_resolve_and_inline(tmpdir):
     # This reproduces the issue reported in
-    # https://github.com/spacetelescope/asdf/issues/406
+    # https://github.com/asdf-format/asdf/issues/406
     tempname = str(tmpdir.join('test.asdf'))
 
     tree = {'random': np.random.random(10)}
@@ -296,6 +335,10 @@ def test_open_pathlib_path(tmpdir):
     with asdf.open(path) as af:
         assert (af['data'] == tree['data']).all()
 
+class FooExtension:
+    types = []
+    tag_mapping = []
+    url_mapping = []
 
 @pytest.mark.parametrize('installed,extension,warns', [
     ('1.2.3', '2.0.0', True),
@@ -305,15 +348,18 @@ def test_open_pathlib_path(tmpdir):
     ('2.0.1', '2.0.dev12345', False),
 ])
 def test_extension_version_check(installed, extension, warns):
+    proxy = ExtensionProxy(FooExtension(), package_name="foo", package_version=installed)
+
+    with config_context() as config:
+        config.add_extension(proxy)
+        af = asdf.AsdfFile()
 
-    af = asdf.AsdfFile()
     af._fname = 'test.asdf'
-    af._extension_metadata['foo.extension.FooExtension'] = ('foo', installed)
 
     tree = {
         'history': {
             'extensions': [
-                asdf.tags.core.ExtensionMetadata(extension_class='foo.extension.FooExtension',
+                asdf.tags.core.ExtensionMetadata(extension_class='asdf.tests.test_api.FooExtension',
                     software=asdf.tags.core.Software(name='foo', version=extension)),
             ]
         }
@@ -330,6 +376,7 @@ def test_extension_version_check(installed, extension, warns):
         af._check_extensions(tree)
 
 
+@pytest.mark.filterwarnings(AsdfDeprecationWarning)
 def test_auto_inline(tmpdir):
     outfile = str(tmpdir.join('test.asdf'))
     tree = {"small_array": np.arange(6), "large_array": np.arange(100)}
@@ -361,61 +408,64 @@ def test_auto_inline(tmpdir):
         assert len(list(af.blocks.internal_blocks)) == 2
 
 
-def test_auto_inline_masked_array(tmpdir):
-    outfile = str(tmpdir.join('test.asdf'))
+@pytest.mark.parametrize("array_inline_threshold, inline_blocks, internal_blocks", [
+    (None, 0, 2),
+    (10, 1, 1),
+    (7, 1, 1),
+    (5, 0, 2),
+    (0, 0, 2),
+    (1, 0, 2),
+])
+def test_array_inline_threshold(array_inline_threshold, inline_blocks, internal_blocks, tmp_path):
+    file_path = tmp_path / "test.asdf"
+    tree = {"small_array": np.arange(6), "large_array": np.arange(100)}
+
+    with asdf.config_context() as config:
+        config.array_inline_threshold = array_inline_threshold
+
+        with asdf.AsdfFile(tree) as af:
+            af.write_to(file_path)
+            assert len(list(af.blocks.inline_blocks)) == inline_blocks
+            assert len(list(af.blocks.internal_blocks)) == internal_blocks
+
+
+@pytest.mark.parametrize("array_inline_threshold, inline_blocks, internal_blocks", [
+    (None, 0, 2),
+    (10, 2, 0),
+    (5, 0, 2),
+])
+def test_array_inline_threshold_masked_array(array_inline_threshold, inline_blocks, internal_blocks, tmp_path):
+    file_path = tmp_path / "test.asdf"
     arr = np.arange(6)
     masked_arr = np.ma.masked_equal(arr, 3)
     tree = {"masked_arr": masked_arr}
 
-    with asdf.AsdfFile(tree) as af:
-        af.write_to(outfile)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 2
+    with asdf.config_context() as config:
+        config.array_inline_threshold = array_inline_threshold
 
-        af.write_to(outfile, auto_inline=10)
-        assert len(list(af.blocks.inline_blocks)) == 2
-        assert len(list(af.blocks.internal_blocks)) == 0
-
-        af.write_to(outfile, auto_inline=5)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 2
+        with asdf.AsdfFile(tree) as af:
+            af.write_to(file_path)
+            assert len(list(af.blocks.inline_blocks)) == inline_blocks
+            assert len(list(af.blocks.internal_blocks)) == internal_blocks
 
 
-def test_auto_inline_large_value(tmpdir):
-    outfile = str(tmpdir.join('test.asdf'))
-    arr = np.array([2**52 + 1, 1, 2, 3, 4, 5])
-    tree = {"array": arr}
-
-    with asdf.AsdfFile(tree) as af:
-        af.write_to(outfile)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 1
-
-        with pytest.raises(ValidationError):
-            af.write_to(outfile, auto_inline=10)
-
-        af.write_to(outfile, auto_inline=5)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 1
-
-
-def test_auto_inline_string_array(tmpdir):
-    outfile = str(tmpdir.join('test.asdf'))
+@pytest.mark.parametrize("array_inline_threshold, inline_blocks, internal_blocks", [
+    (None, 0, 1),
+    (10, 1, 0),
+    (5, 0, 1),
+])
+def test_array_inline_threshold_string_array(array_inline_threshold, inline_blocks, internal_blocks, tmp_path):
+    file_path = tmp_path / "test.asdf"
     arr = np.array(["peach", "plum", "apricot", "nectarine", "cherry", "pluot"])
     tree = {"array": arr}
 
-    with asdf.AsdfFile(tree) as af:
-        af.write_to(outfile)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 1
+    with asdf.config_context() as config:
+        config.array_inline_threshold = array_inline_threshold
 
-        af.write_to(outfile, auto_inline=10)
-        assert len(list(af.blocks.inline_blocks)) == 1
-        assert len(list(af.blocks.internal_blocks)) == 0
-
-        af.write_to(outfile, auto_inline=5)
-        assert len(list(af.blocks.inline_blocks)) == 0
-        assert len(list(af.blocks.internal_blocks)) == 1
+        with asdf.AsdfFile(tree) as af:
+            af.write_to(file_path)
+            assert len(list(af.blocks.inline_blocks)) == inline_blocks
+            assert len(list(af.blocks.internal_blocks)) == internal_blocks
 
 
 def test_resolver_deprecations():
@@ -493,6 +543,7 @@ def test_info_module(capsys, tmpdir):
         for val in tree["foo"][i-1:]:
             assert val not in captured.out
 
+
 def test_info_asdf_file(capsys, tmpdir):
     tree = dict(
         foo=42, bar="hello", baz=np.arange(20),
@@ -507,6 +558,51 @@ def test_info_asdf_file(capsys, tmpdir):
     assert "baz" in captured.out
 
 
+class ObjectWithInfoSupport:
+
+    def __init__(self):
+        self._tag = "foo"
+
+    def __asdf_traverse__(self):
+        return {'the_meaning_of_life_the_universe_and_everything': 42,
+                'clown': 'Bozo'}
+
+
+def test_info_object_support(capsys):
+    tree = dict(random=3.14159, object=ObjectWithInfoSupport())
+    af = asdf.AsdfFile(tree)
+    af.info()
+    captured = capsys.readouterr()
+    assert "the_meaning_of_life_the_universe_and_everything" in captured.out
+    assert "clown" in captured.out
+    assert "42" in captured.out
+    assert "Bozo" in captured.out
+
+
+class RecursiveObjectWithInfoSupport:
+
+    def __init__(self):
+        self._tag = "foo"
+        self.the_meaning = 42
+        self.clown = "Bozo"
+        self.recursive = None
+
+    def __asdf_traverse__(self):
+        return {'the_meaning': self.the_meaning,
+                'clown': self.clown,
+                'recursive': self.recursive}
+
+
+def test_recursive_info_object_support(capsys):
+    recursive_obj = RecursiveObjectWithInfoSupport()
+    recursive_obj.recursive = recursive_obj
+    tree = dict(random=3.14159, rtest=recursive_obj)
+    af = asdf.AsdfFile(tree)
+    af.info()
+    captured = capsys.readouterr()
+    assert "recursive reference" in captured.out
+
+
 def test_search():
     tree = dict(foo=42, bar="hello", baz=np.arange(20))
     af = asdf.AsdfFile(tree)
@@ -555,3 +651,13 @@ def test_array_access_after_file_close(tmpdir):
     with asdf.open(path, lazy_load=False, copy_arrays=True) as af:
         tree = af.tree
     assert_array_equal(tree["data"], data)
+
+
+def test_none_values(tmpdir):
+    path = str(tmpdir.join("test.asdf"))
+
+    af = asdf.AsdfFile({"foo": None})
+    af.write_to(path)
+    with asdf.open(path) as af:
+        assert "foo" in af
+        assert af["foo"] is None
