@@ -1,7 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
-
 import copy
 import os
 import pytest
@@ -16,6 +12,7 @@ from astropy.table import Table
 from jsonschema.exceptions import ValidationError
 
 import asdf
+from asdf import get_config
 from asdf import fits_embed
 from asdf import open as asdf_open
 from asdf.exceptions import AsdfWarning, AsdfConversionWarning
@@ -28,7 +25,7 @@ from .helpers import (
 )
 
 
-TEST_DTYPES = ['<f8', '<f8', '<u4', '>u4', '<i4', '>i4']
+TEST_DTYPES = ['<f8', '>f8', '<u4', '>u4', '<i4', '>i4']
 
 
 def create_asdf_in_fits(dtype):
@@ -299,10 +296,6 @@ def test_asdf_open(tmpdir, dtype):
         with asdf_open(handle) as ff:
             compare_asdfs(asdf_in_fits, ff)
 
-    # Test opening the file as a FITS file first and passing the HDUList
-    with fits.open(tmpfile) as hdulist:
-        with asdf_open(hdulist) as ff:
-            compare_asdfs(asdf_in_fits, ff)
 
 def test_validate_on_read(tmpdir):
     tmpfile = str(tmpdir.join('invalid.fits'))
@@ -323,10 +316,12 @@ invalid_software: !core/software-1.0.0
 
     for open_method in [asdf.open, fits_embed.AsdfInFits.open]:
         with pytest.raises(ValidationError):
-            with open_method(tmpfile, validate_on_read=True):
+            get_config().validate_on_read = True
+            with open_method(tmpfile):
                 pass
 
-        with open_method(tmpfile, validate_on_read=False) as af:
+        get_config().validate_on_read = False
+        with open_method(tmpfile) as af:
             assert af["invalid_software"]["name"] == "Minesweeper"
             assert af["invalid_software"]["version"] == 3
 
@@ -334,17 +329,11 @@ invalid_software: !core/software-1.0.0
 def test_open_gzipped():
     testfile = get_test_data_path('asdf.fits.gz')
 
-    # Opening as an HDU should work
-    with fits.open(testfile) as ff:
-        with asdf.open(ff) as af:
-            assert af.tree['stuff'].shape == (20, 20)
-
     with fits_embed.AsdfInFits.open(testfile) as af:
         assert af.tree['stuff'].shape == (20, 20)
 
-    with asdf.open(testfile) as af:
-        assert af.tree['stuff'].shape == (20, 20)
 
+@pytest.mark.filterwarnings('ignore::astropy.io.fits.verify.VerifyWarning')
 def test_bad_input(tmpdir):
     """Make sure these functions behave properly with bad input"""
     text_file = os.path.join(str(tmpdir), 'test.txt')
@@ -397,10 +386,11 @@ def test_serialize_table(tmpdir):
         data = ff.tree['my_table']
         assert data._source.startswith('fits:')
 
+
 def test_extension_check():
     testfile = get_test_data_path('extension_check.fits')
 
-    with pytest.warns(AsdfWarning, match="was created with extension 'foo.bar.FooBar'"):
+    with pytest.warns(AsdfWarning, match="was created with extension class 'foo.bar.FooBar'"):
         with asdf.open(testfile):
             pass
 
@@ -424,6 +414,7 @@ def test_verify_with_astropy(tmpdir, dtype):
     with fits.open(tmpfile) as hdu:
         hdu.verify('exception')
 
+
 def test_dangling_file_handle(tmpdir):
     """
     This tests the bug fix introduced in #533. Without the bug fix, this test
@@ -457,3 +448,78 @@ def test_dangling_file_handle(tmpdir):
     gc.collect()
 
     del ctx
+
+
+def test_array_view(tmp_path):
+    """
+    Special handling is required when a view over a larger array
+    is assigned to an HDU and referenced from the ASDF tree.
+    """
+    file_path = tmp_path / "test.fits"
+
+    data = np.arange(100, dtype=np.float64).reshape(5, 20)
+    data_view = data[:, :10]
+
+    hdul = fits.HDUList([fits.PrimaryHDU(), fits.ImageHDU(data_view)])
+    with asdf.fits_embed.AsdfInFits(hdulist=hdul) as af:
+        af["data"] = hdul[-1].data
+        af.write_to(file_path)
+
+    with asdf.open(file_path) as af:
+        assert_array_equal(af["data"], data_view)
+
+
+def test_array_view_compatible_layout(tmp_path):
+    """
+    We should be able to serialize additional views that have
+    the same memory layout.
+    """
+    file_path = tmp_path / "test.fits"
+
+    data = np.arange(100, dtype=np.float64).reshape(5, 20)
+    data_view = data[:, :10]
+    other_view = data_view[:, :]
+
+    hdul = fits.HDUList([fits.PrimaryHDU(), fits.ImageHDU(data_view)])
+    with asdf.fits_embed.AsdfInFits(hdulist=hdul) as af:
+        af["data"] = hdul[-1].data
+        af["other"] = other_view
+        af.write_to(file_path)
+
+    with asdf.open(file_path) as af:
+        assert_array_equal(af["data"], data_view)
+        assert_array_equal(af["other"], other_view)
+
+
+def test_array_view_compatible_dtype(tmp_path):
+    """
+    Changing the dtype of a view over a FITS array is prohibited.
+    """
+    file_path = tmp_path / "test.fits"
+
+    data = np.arange(100, dtype=np.float64)
+    hdul = fits.HDUList([fits.PrimaryHDU(), fits.ImageHDU(data)])
+    with pytest.raises(ValueError, match="ASDF has only limited support for serializing views over arrays stored in FITS HDUs"):
+        with asdf.fits_embed.AsdfInFits(hdulist=hdul) as af:
+            af["view"] = hdul[-1].data.view(np.int64)
+            af.write_to(file_path)
+
+
+def test_array_view_different_layout(tmp_path):
+    """
+    A view over the FITS array with a different memory layout
+    might end up corrupted when astropy.io.fits changes the
+    array to C-contiguous and big-endian on write.
+    """
+    file_path = tmp_path / "test.fits"
+
+    data = np.arange(100, dtype=np.float64).reshape(5, 20)
+    data_view = data[:, :10]
+    other_view = data_view[:, 10:]
+
+    hdul = fits.HDUList([fits.PrimaryHDU(), fits.ImageHDU(data_view)])
+    with asdf.fits_embed.AsdfInFits(hdulist=hdul) as af:
+        af["data"] = hdul[-1].data
+        af["other"] = other_view
+        with pytest.raises(ValueError, match="ASDF has only limited support for serializing views over arrays stored in FITS HDUs"):
+            af.write_to(file_path)
