@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import io
 import os
 import sys
@@ -12,9 +9,10 @@ import urllib.request as urllib_request
 import numpy as np
 
 import asdf
+from asdf import exceptions
 from asdf import util
 from asdf import generic_io
-from asdf.asdf import is_asdf_file
+from asdf.config import config_context
 
 from . import helpers, create_small_tree, create_large_tree
 
@@ -232,7 +230,7 @@ def test_streams2():
 
     fd = generic_io.InputStream(buff, 'r')
 
-    x = fd._peek(10)
+    x = fd.peek(10)
     x = fd.read()
     assert len(x) == 60
 
@@ -264,7 +262,6 @@ def test_http_connection(tree, httpserver):
 
     def get_read_fd():
         fd = generic_io.get_file(httpserver.url + "test.asdf")
-        assert isinstance(fd, generic_io.InputStream)
         # This is to check for a "feature" in Python 3.x that reading zero
         # bytes from a socket causes it to stop.  We have code in generic_io.py
         # to workaround it.
@@ -273,34 +270,6 @@ def test_http_connection(tree, httpserver):
 
     with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
         assert len(list(ff.blocks.internal_blocks)) == 2
-        assert not isinstance(next(ff.blocks.internal_blocks)._data, np.core.memmap)
-        assert isinstance(next(ff.blocks.internal_blocks)._data, np.ndarray)
-        ff.tree['science_data'][0] == 42
-
-
-#@pytest.mark.remote_data
-@pytest.mark.skip(reason="The _roundtrip assert isn't well defined and returning different lengths in different environments.")
-def test_http_connection_range(tree, rhttpserver):
-    path = os.path.join(rhttpserver.tmpdir, 'test.asdf')
-    connection = [None]
-
-    def get_write_fd():
-        return generic_io.get_file(open(path, 'wb'), mode='w')
-
-    def get_read_fd():
-        fd = generic_io.get_file(rhttpserver.url + "test.asdf")
-        assert isinstance(fd, generic_io.HTTPConnection)
-        connection[0] = fd
-        return fd
-
-    with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
-        if len(tree) == 4:
-            assert connection[0]._nreads == 0
-        else:
-            assert connection[0]._nreads == 6
-
-        assert len(list(ff.blocks.internal_blocks)) == 2
-        assert isinstance(next(ff.blocks.internal_blocks)._data, np.core.memmap)
         assert isinstance(next(ff.blocks.internal_blocks)._data, np.ndarray)
         ff.tree['science_data'][0] == 42
 
@@ -457,6 +426,22 @@ def test_relative_uri():
         'http://www.google.com', 'file://local') == 'file://local'
 
 
+@pytest.mark.parametrize("protocol", ["http", "asdf"])
+def test_resolve_uri(protocol):
+    """
+    Confirm that the patched urllib.parse is handling
+    asdf:// URIs correctly.
+    """
+    assert generic_io.resolve_uri(
+        '{}://somewhere.org/some-schema'.format(protocol), '#/definitions/foo'
+    ) == '{}://somewhere.org/some-schema#/definitions/foo'.format(protocol)
+
+    assert generic_io.resolve_uri(
+        '{}://somewhere.org/path/to/some-schema'.format(protocol),
+        '../../some/other/path/to/some-other-schema'
+    ) == '{}://somewhere.org/some/other/path/to/some-other-schema'.format(protocol)
+
+
 def test_arbitrary_file_object():
     class Wrapper:
         def __init__(self, init):
@@ -526,7 +511,7 @@ def test_truncated_reader():
     Tests several edge cases for _TruncatedReader.read()
 
     Includes regression test for
-    https://github.com/spacetelescope/asdf/pull/181
+    https://github.com/asdf-format/asdf/pull/181
     """
 
     # TODO: Should probably break this up into multiple test cases
@@ -538,14 +523,14 @@ def test_truncated_reader():
 
     # Simple cases where the delimiter is not found at all
     tr = generic_io._TruncatedReader(fd, b'x', 1)
-    with pytest.raises(ValueError):
+    with pytest.raises(exceptions.DelimiterNotFoundError):
         tr.read()
 
     fd.seek(0)
     tr = generic_io._TruncatedReader(fd, b'x', 1)
     assert tr.read(100) == content[:100]
     assert tr.read(1) == content[100:]
-    with pytest.raises(ValueError):
+    with pytest.raises(exceptions.DelimiterNotFoundError):
         tr.read()
 
     fd.seek(0)
@@ -763,16 +748,19 @@ def test_truncated_reader():
     assert tr.read() == b''
 
 
-def test_is_asdf(tmpdir):
-    # test fits
-    fits = pytest.importorskip('astropy.io.fits')
+def test_blocksize(tree, tmpdir):
+    path = os.path.join(str(tmpdir), 'test.asdf')
 
-    hdul = fits.HDUList()
-    phdu= fits.PrimaryHDU()
-    imhdu= fits.ImageHDU(data=np.arange(24).reshape((4,6)))
-    hdul.append(phdu)
-    hdul.append(imhdu)
-    path = os.path.join(str(tmpdir), 'test.fits')
-    hdul.writeto(path)
-    assert not is_asdf_file(path)
-    assert is_asdf_file(asdf.AsdfFile())
+    def get_write_fd():
+        f = generic_io.get_file(open(path, 'wb'), mode='w', close=True)
+        return f
+
+    def get_read_fd():
+        # Must open with mode=rw in order to get memmapped data
+        f = generic_io.get_file(open(path, 'r+b'), mode='rw', close=True)
+        return f
+
+    with config_context() as config:
+        config.io_block_size = 1233  # make sure everything works with a strange blocksize
+        with _roundtrip(tree, get_write_fd, get_read_fd) as ff:
+            assert ff._fd.block_size == 1233
