@@ -1,7 +1,5 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import io
+from datetime import datetime
 
 from jsonschema import ValidationError
 import numpy as np
@@ -9,6 +7,8 @@ from numpy.testing import assert_array_equal
 import pytest
 
 import asdf
+from asdf import constants
+from asdf import get_config, config_context
 from asdf import extension
 from asdf import resolver
 from asdf import schema
@@ -17,7 +17,8 @@ from asdf import util
 from asdf import yamlutil
 from asdf import tagged
 from asdf.tests import helpers, CustomExtension
-from asdf.exceptions import AsdfWarning, AsdfConversionWarning
+from asdf.exceptions import AsdfWarning, AsdfConversionWarning, AsdfDeprecationWarning
+
 
 
 class TagReferenceType(types.CustomType):
@@ -64,7 +65,7 @@ def test_read_json_schema():
     """Pytest to make sure reading JSON schemas succeeds.
 
     This was known to fail on Python 3.5 See issue #314 at
-    https://github.com/spacetelescope/asdf/issues/314 for more details.
+    https://github.com/asdf-format/asdf/issues/314 for more details.
     """
     json_schema = helpers.get_test_data_path('example_schema.json')
     schema_tree = schema.load_schema(json_schema, resolve_references=True)
@@ -165,6 +166,91 @@ required: [foobar]
     schema.check_schema(schema_tree)
 
 
+def test_load_schema_with_asdf_uri_scheme():
+    subschema_content="""%YAML 1.1
+---
+$schema: http://stsci.edu/schemas/asdf/asdf-schema-1.0.0
+id: asdf://somewhere.org/schemas/bar
+
+bar:
+  type: string
+...
+"""
+    content = """%YAML 1.1
+---
+$schema: http://stsci.edu/schemas/asdf/asdf-schema-1.0.0
+id: asdf://somewhere.org/schemas/foo
+
+definitions:
+  local_bar:
+    type: string
+
+type: object
+properties:
+  bar:
+    $ref: asdf://somewhere.org/schemas/bar#/bar
+  local_bar:
+    $ref: '#/definitions/local_bar'
+...
+"""
+    with asdf.config_context() as config:
+        config.add_resource_mapping({"asdf://somewhere.org/schemas/foo": content})
+        config.add_resource_mapping({"asdf://somewhere.org/schemas/bar": subschema_content})
+
+        schema_tree = schema.load_schema("asdf://somewhere.org/schemas/foo")
+        instance = {"bar": "baz", "local_bar": "foz"}
+        schema.validate(instance, schema=schema_tree)
+        with pytest.raises(ValidationError):
+            schema.validate({"bar": 12}, schema=schema_tree)
+
+
+def test_load_schema_with_stsci_id():
+    """
+    This tests the following edge case:
+    - schema references a subschema provided by the new extension API
+    - subschema URI shares a prefix with one of the old-style extension resolvers
+    - resolve_references is enabled
+
+    If we're not careful, the old-style resolver will mangle the URI and
+    we won't be able to retrieve the schema content.
+    """
+    subschema_content="""%YAML 1.1
+---
+$schema: http://stsci.edu/schemas/asdf/asdf-schema-1.0.0
+id: http://stsci.edu/schemas/bar
+
+bar:
+  type: string
+...
+"""
+    content = """%YAML 1.1
+---
+$schema: http://stsci.edu/schemas/asdf/asdf-schema-1.0.0
+id: http://stsci.edu/schemas/foo
+
+definitions:
+  local_bar:
+    type: string
+
+type: object
+properties:
+  bar:
+    $ref: http://stsci.edu/schemas/bar#/bar
+  local_bar:
+    $ref: '#/definitions/local_bar'
+...
+"""
+    with asdf.config_context() as config:
+        config.add_resource_mapping({"http://stsci.edu/schemas/foo": content})
+        config.add_resource_mapping({"http://stsci.edu/schemas/bar": subschema_content})
+
+        schema_tree = schema.load_schema("http://stsci.edu/schemas/foo", resolve_references=True)
+        instance = {"bar": "baz", "local_bar": "foz"}
+        schema.validate(instance, schema=schema_tree)
+        with pytest.raises(ValidationError):
+            schema.validate({"bar": 12}, schema=schema_tree)
+
+
 def test_schema_caching():
     # Make sure that if we request the same URL, we get a different object
     # (despite the caching internal to load_schema).  Changes to a schema
@@ -187,6 +273,24 @@ def test_asdf_file_resolver_hashing():
     assert a1.resolver == a2.resolver
 
 
+def test_load_schema_from_resource_mapping():
+    content = """
+id: http://somewhere.org/schemas/razmataz-1.0.0
+type: object
+properties:
+  foo:
+    type: string
+  bar:
+    type: boolean
+""".encode("utf-8")
+
+    get_config().add_resource_mapping({"http://somewhere.org/schemas/razmataz-1.0.0": content})
+
+    s = schema.load_schema("http://somewhere.org/schemas/razmataz-1.0.0")
+
+    assert s["id"] == "http://somewhere.org/schemas/razmataz-1.0.0"
+
+
 def test_flow_style():
     class CustomFlowStyleType(dict, types.CustomType):
         name = 'custom_flow'
@@ -407,42 +511,56 @@ custom: !<tag:nowhere.org:custom/default-1.0.0>
         assert ff.tree['custom']['j']['l'] == 362
 
     buff.seek(0)
-    with asdf.open(buff, extensions=[DefaultTypeExtension()],
-                            do_not_fill_defaults=True) as ff:
-        assert 'a' not in ff.tree['custom']
-        assert 'c' not in ff.tree['custom']['b']
-        assert 'e' not in ff.tree['custom']['d']
-        assert 'f' not in ff.tree['custom']['d']
-        assert 'h' not in ff.tree['custom']['g']
-        assert 'i' not in ff.tree['custom']['g']
-        assert 'k' not in ff.tree['custom']['j']
-        assert ff.tree['custom']['j']['l'] == 362
-        ff.fill_defaults()
-        assert 'a' in ff.tree['custom']
-        assert ff.tree['custom']['a'] == 42
-        assert 'c' in ff.tree['custom']['b']
-        assert ff.tree['custom']['b']['c'] == 82
-        assert ff.tree['custom']['b']['c'] == 82
-        assert ff.tree['custom']['d']['e'] == 122
-        assert ff.tree['custom']['d']['f'] == 162
-        assert 'h' not in ff.tree['custom']['g']
-        assert 'i' not in ff.tree['custom']['g']
-        assert 'k' not in ff.tree['custom']['j']
-        assert ff.tree['custom']['j']['l'] == 362
-        ff.remove_defaults()
-        assert 'a' not in ff.tree['custom']
-        assert 'c' not in ff.tree['custom']['b']
-        assert 'e' not in ff.tree['custom']['d']
-        assert 'f' not in ff.tree['custom']['d']
-        assert 'h' not in ff.tree['custom']['g']
-        assert 'i' not in ff.tree['custom']['g']
-        assert 'k' not in ff.tree['custom']['j']
-        assert ff.tree['custom']['j']['l'] == 362
+    with pytest.warns(AsdfDeprecationWarning, match='do_not_fill_defaults'):
+        with asdf.open(buff, extensions=[DefaultTypeExtension()],
+                                do_not_fill_defaults=True) as ff:
+            assert 'a' not in ff.tree['custom']
+            assert 'c' not in ff.tree['custom']['b']
+            assert 'e' not in ff.tree['custom']['d']
+            assert 'f' not in ff.tree['custom']['d']
+            assert 'h' not in ff.tree['custom']['g']
+            assert 'i' not in ff.tree['custom']['g']
+            assert 'k' not in ff.tree['custom']['j']
+            assert ff.tree['custom']['j']['l'] == 362
+            ff.fill_defaults()
+            assert 'a' in ff.tree['custom']
+            assert ff.tree['custom']['a'] == 42
+            assert 'c' in ff.tree['custom']['b']
+            assert ff.tree['custom']['b']['c'] == 82
+            assert ff.tree['custom']['b']['c'] == 82
+            assert ff.tree['custom']['d']['e'] == 122
+            assert ff.tree['custom']['d']['f'] == 162
+            assert 'h' not in ff.tree['custom']['g']
+            assert 'i' not in ff.tree['custom']['g']
+            assert 'k' not in ff.tree['custom']['j']
+            assert ff.tree['custom']['j']['l'] == 362
+            ff.remove_defaults()
+            assert 'a' not in ff.tree['custom']
+            assert 'c' not in ff.tree['custom']['b']
+            assert 'e' not in ff.tree['custom']['d']
+            assert 'f' not in ff.tree['custom']['d']
+            assert 'h' not in ff.tree['custom']['g']
+            assert 'i' not in ff.tree['custom']['g']
+            assert 'k' not in ff.tree['custom']['j']
+            assert ff.tree['custom']['j']['l'] == 362
+
+    buff.seek(0)
+    with config_context() as config:
+        config.legacy_fill_schema_defaults = False
+        with asdf.open(buff, extensions=[DefaultTypeExtension()]) as ff:
+            assert 'a' not in ff.tree['custom']
+            assert 'c' not in ff.tree['custom']['b']
+            assert 'e' not in ff.tree['custom']['d']
+            assert 'f' not in ff.tree['custom']['d']
+            assert 'h' not in ff.tree['custom']['g']
+            assert 'i' not in ff.tree['custom']['g']
+            assert 'k' not in ff.tree['custom']['j']
+            assert ff.tree['custom']['j']['l'] == 362
 
 
 def test_one_of():
     """
-    Covers https://github.com/spacetelescope/asdf/issues/809
+    Covers https://github.com/asdf-format/asdf/issues/809
     """
     class OneOfType(dict, types.CustomType):
         name = 'one_of'
@@ -548,46 +666,54 @@ def test_schema_resolved_via_entry_points():
     assert tag in repr(s)
 
 
-@pytest.mark.parametrize('use_numpy', [False, True])
-def test_large_literals(use_numpy):
-
-    largeval = 1 << 53
-    if use_numpy:
-        largeval = np.uint64(largeval)
+@pytest.mark.parametrize("num", [constants.MAX_NUMBER+1, constants.MIN_NUMBER-1])
+def test_max_min_literals(num):
 
     tree = {
-        'large_int': largeval,
+        'test_int': num,
     }
 
     with pytest.raises(ValidationError):
         asdf.AsdfFile(tree)
 
     tree = {
-        'large_list': [largeval],
+        'test_list': [num],
     }
 
     with pytest.raises(ValidationError):
         asdf.AsdfFile(tree)
 
     tree = {
-        'large_array': np.array([largeval], np.uint64)
+        num: 'test_key',
     }
 
-    ff = asdf.AsdfFile(tree)
-    buff = io.BytesIO()
-    ff.write_to(buff)
-
-    ff.set_array_storage(ff.tree['large_array'], 'inline')
-    buff = io.BytesIO()
     with pytest.raises(ValidationError):
-        ff.write_to(buff)
-        print(buff.getvalue())
+        asdf.AsdfFile(tree)
 
 
-def test_read_large_literal():
+@pytest.mark.parametrize("num", [constants.MAX_NUMBER+1, constants.MIN_NUMBER-1])
+@pytest.mark.parametrize("ttype", ["val", "list", "key"])
+def test_max_min_literals_write(num, ttype, tmpdir):
+    outfile = tmpdir / "test.asdf"
+    af = asdf.AsdfFile()
 
-    value = 1 << 64
-    yaml = """integer: {}""".format(value)
+    # Validation doesn't occur here, so no warning/error will be raised.
+    if ttype == "val":
+        af.tree['test_int'] = num
+    elif ttype == "list":
+        af.tree['test_int'] = [num]
+    else:
+        af.tree[num] = 'test_key'
+
+    # Validation will occur on write, though, so detect it.
+    with pytest.raises(ValidationError):
+        af.write_to(outfile)
+    af.close()
+
+
+@pytest.mark.parametrize("value", [constants.MAX_NUMBER+1, constants.MIN_NUMBER-1])
+def test_read_large_literal(value):
+    yaml = f"integer: {value}"
 
     buff = helpers.yaml_to_asdf(yaml)
 
@@ -595,6 +721,46 @@ def test_read_large_literal():
         with asdf.open(buff) as af:
             assert af['integer'] == value
 
+    yaml = f"{value}: foo"
+
+    buff = helpers.yaml_to_asdf(yaml)
+
+    with pytest.warns(AsdfWarning, match="Invalid integer literal value"):
+        with asdf.open(buff) as af:
+            assert af[value] == "foo"
+
+
+@pytest.mark.parametrize(
+    "version,keys",
+    [
+        ("1.6.0", ["foo", 42, True]),
+        ("1.5.0", ["foo", 42, True, 3.14159, datetime.now(), b"foo", None]),
+    ]
+)
+def test_mapping_supported_key_types(keys, version):
+    for key in keys:
+        with helpers.assert_no_warnings():
+            af = asdf.AsdfFile({key: "value"}, version=version)
+            buff = io.BytesIO()
+            af.write_to(buff)
+            buff.seek(0)
+            with asdf.open(buff) as af:
+                assert af[key] == "value"
+
+
+@pytest.mark.parametrize(
+    "version,keys",
+    [
+        ("1.6.0", [3.14159, datetime.now(), b"foo", None, ("foo", "bar")]),
+    ]
+)
+def test_mapping_unsupported_key_types(keys, version):
+    for key in keys:
+        with pytest.raises(ValidationError, match="Mapping key .* is not permitted"):
+            af = asdf.AsdfFile({key: "value"}, version=version)
+            buff = io.BytesIO()
+            af.write_to(buff)
+
 
 def test_nested_array():
     s = {
@@ -777,7 +943,7 @@ def test_custom_validation_pathlib(tmpdir):
     """
     Make sure custom schema paths can be pathlib.Path objects
 
-    See https://github.com/spacetelescope/asdf/issues/653 for discussion.
+    See https://github.com/asdf-format/asdf/issues/653 for discussion.
     """
     from pathlib import Path
 
@@ -1004,3 +1170,37 @@ def test_validator_visit_repeat_nodes():
     )
     validator.validate(tree)
     assert len(visited_nodes) == 3
+
+
+def test_tag_validator():
+    content="""%YAML 1.1
+---
+$schema: http://stsci.edu/schemas/asdf/asdf-schema-1.0.0
+id: asdf://somewhere.org/schemas/foo
+tag: asdf://somewhere.org/tags/foo
+...
+"""
+    with asdf.config_context() as config:
+        config.add_resource_mapping({"asdf://somewhere.org/schemas/foo": content})
+
+        schema_tree = schema.load_schema("asdf://somewhere.org/schemas/foo")
+        instance = tagged.TaggedDict(tag="asdf://somewhere.org/tags/foo")
+        schema.validate(instance, schema=schema_tree)
+        with pytest.raises(ValidationError):
+            schema.validate(tagged.TaggedDict(tag="asdf://somewhere.org/tags/bar"), schema=schema_tree)
+
+    content="""%YAML 1.1
+---
+$schema: http://stsci.edu/schemas/asdf/asdf-schema-1.0.0
+id: asdf://somewhere.org/schemas/bar
+tag: asdf://somewhere.org/tags/bar-*
+...
+"""
+    with asdf.config_context() as config:
+        config.add_resource_mapping({"asdf://somewhere.org/schemas/bar": content})
+
+        schema_tree = schema.load_schema("asdf://somewhere.org/schemas/bar")
+        instance = tagged.TaggedDict(tag="asdf://somewhere.org/tags/bar-2.5")
+        schema.validate(instance, schema=schema_tree)
+        with pytest.raises(ValidationError):
+            schema.validate(tagged.TaggedDict(tag="asdf://somewhere.org/tags/foo-1.0"), schema=schema_tree)
