@@ -1,6 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
 import io
 import os
 from fractions import Fraction
@@ -388,7 +385,7 @@ undefined_data:
         - !core/complex-1.0.0 3.14j
 """
     buff = helpers.yaml_to_asdf(yaml)
-    with pytest.warns(None) as warning:
+    with pytest.warns(Warning) as warning:
         afile = asdf.open(buff)
         missing = afile.tree['undefined_data']
 
@@ -607,53 +604,6 @@ flow_thing:
         asdf.open(buff, extensions=CustomFlowExtension())
 
 
-def test_extension_override(tmpdir):
-
-    gwcs = pytest.importorskip('gwcs', '0.12.0')
-
-    from asdf.extension import default_extensions
-    default_extensions.reset()
-
-    version = str(versioning.default_version)
-    tmpfile = str(tmpdir.join('override.asdf'))
-
-    with asdf.AsdfFile() as aa:
-        assert aa.type_index.from_custom_type(gwcs.WCS, version=version) is gwcs.tags.wcs.WCSType
-        aa.tree['wcs'] = gwcs.WCS(output_frame='icrs')
-        aa.write_to(tmpfile)
-
-    with open(tmpfile, 'rb') as ff:
-        contents = str(ff.read())
-        assert gwcs.tags.wcs.WCSType.yaml_tag in contents
-
-
-def test_extension_override_subclass(tmpdir):
-
-    gwcs = pytest.importorskip('gwcs', '0.12.0')
-    pytest.importorskip('astropy', '4.0.0')
-
-    from asdf.extension import default_extensions
-    default_extensions.reset()
-
-    version = str(versioning.default_version)
-    tmpfile = str(tmpdir.join('override.asdf'))
-
-    class SubclassWCS(gwcs.WCS):
-        pass
-
-    with asdf.AsdfFile() as aa:
-        assert aa.type_index.from_custom_type(gwcs.WCS, version=version) is gwcs.tags.wcs.WCSType
-        assert aa.type_index.from_custom_type(SubclassWCS, version=version) is gwcs.tags.wcs.WCSType
-        # The duplication here is deliberate: make sure that nothing has changed
-        assert aa.type_index.from_custom_type(gwcs.WCS, version=version) is gwcs.tags.wcs.WCSType
-        aa.tree['wcs'] = SubclassWCS(output_frame='icrs')
-        aa.write_to(tmpfile)
-
-    with open(tmpfile, 'rb') as ff:
-        contents = str(ff.read())
-        assert gwcs.tags.wcs.WCSType.yaml_tag in contents
-
-
 def test_tag_without_schema(tmpdir):
 
     tmpfile = str(tmpdir.join('foo.asdf'))
@@ -702,156 +652,6 @@ def test_tag_without_schema(tmpdir):
             assert ff.tree['foo'] == tree['foo']
 
 
-def test_subclass_decorator(tmpdir):
-
-    tmpfile = str(tmpdir.join('subclass.asdf'))
-
-    (FractionType, Fractional2dCoordType,
-        Fractional2dCoordExtension) = fractional2dcoordtype_factory()
-
-    extension = Fractional2dCoordExtension()
-
-    coord = Fractional2dCoord(Fraction(2, 3), Fraction(7, 9))
-    tree = dict(coord=coord)
-
-    # First make sure the base type is serialized properly
-    with asdf.AsdfFile(tree, extensions=extension) as af:
-        af.write_to(tmpfile)
-
-    with asdf.open(tmpfile, extensions=extension) as af:
-        assert isinstance(af['coord'], Fractional2dCoord)
-        assert af['coord'].x == coord.x
-        assert af['coord'].y == coord.y
-
-    # Now create a subclass
-    @Fractional2dCoordType.subclass
-    class Subclass2dCoord(Fractional2dCoord):
-        pass
-
-    subclass_coord = Subclass2dCoord(Fraction(2, 3), Fraction(7, 9))
-    tree = dict(coord=subclass_coord)
-
-    with asdf.AsdfFile(tree, extensions=extension) as af:
-        af.write_to(tmpfile)
-
-    with asdf.open(tmpfile, extensions=extension) as af:
-        assert isinstance(af['coord'], Subclass2dCoord)
-        assert af['coord'].x == subclass_coord.x
-        assert af['coord'].y == subclass_coord.y
-
-
-def test_subclass_decorator_custom_attribute(tmpdir):
-
-    tmpfile = str(tmpdir.join('subclass.asdf'))
-
-    (FractionType, Fractional2dCoordType,
-        Fractional2dCoordExtension) = fractional2dcoordtype_factory()
-
-    extension = Fractional2dCoordExtension()
-
-    coord = Fractional2dCoord(Fraction(2, 3), Fraction(7, 9))
-    tree = dict(coord=coord)
-
-    # First make sure the base type is serialized properly
-    with asdf.AsdfFile(tree, extensions=extension) as af:
-        af.write_to(tmpfile)
-
-    with asdf.open(tmpfile, extensions=extension) as af:
-        assert isinstance(af['coord'], Fractional2dCoord)
-        assert af['coord'].x == coord.x
-        assert af['coord'].y == coord.y
-
-    # Now create a subclass
-    @Fractional2dCoordType.subclass(attribute='bizbaz')
-    class Subclass2dCoord(Fractional2dCoord):
-        pass
-
-    subclass_coord = Subclass2dCoord(Fraction(2, 3), Fraction(7, 9))
-    tree = dict(coord=subclass_coord)
-
-    with asdf.AsdfFile(tree, extensions=extension) as af:
-        af.write_to(tmpfile)
-
-    tmp = asdf.AsdfFile()
-    af_yaml = asdf.AsdfFile._open_asdf(tmp, tmpfile, _force_raw_types=True)
-    assert 'bizbaz' in af_yaml['coord']
-    assert 'Subclass2dCoord' in af_yaml['coord']['bizbaz']['name']
-    af_yaml.close()
-
-    with asdf.open(tmpfile, extensions=extension) as af:
-        assert isinstance(af['coord'], Subclass2dCoord)
-        assert af['coord'].x == subclass_coord.x
-        assert af['coord'].y == subclass_coord.y
-
-
-def test_subclass_decorator_attribute(tmpdir):
-
-    tmpfile = str(tmpdir.join('subclass.asdf'))
-
-    (FractionType, Fractional2dCoordType,
-        Fractional2dCoordExtension) = fractional2dcoordtype_factory()
-
-    extension = Fractional2dCoordExtension()
-
-    @Fractional2dCoordType.subclass
-    class Subclass2dCoord(Fractional2dCoord):
-        def __init__(self, *args, custom=None, other=None, **kwargs):
-            super().__init__(*args, **kwargs)
-            self._custom = custom
-            self._other = other
-
-        @Fractional2dCoordType.subclass_property
-        def custom(self):
-            return self._custom
-
-        @Fractional2dCoordType.subclass_property
-        def other(self):
-            return self._other
-
-    subclass_coord = Subclass2dCoord(Fraction(2, 3), Fraction(7, 9),
-                                     custom='testing', other=[1,2,3,4])
-    tree = dict(coord=subclass_coord)
-
-    with asdf.AsdfFile(tree, extensions=extension) as af:
-        af.write_to(tmpfile)
-
-    with asdf.open(tmpfile, extensions=extension) as af:
-        assert isinstance(af['coord'], Subclass2dCoord)
-        assert af['coord'].x == subclass_coord.x
-        assert af['coord'].y == subclass_coord.y
-        assert af['coord'].custom == 'testing'
-        assert af['coord'].other == [1,2,3,4]
-
-
-def test_subclass_decorator_warning():
-
-    FractionType = fractiontype_factory()
-
-    class FractionExtension(CustomExtension):
-        @property
-        def types(self):
-            return [FractionType]
-
-    @FractionType.subclass
-    class MyFraction(Fraction):
-        # We need to override __new__ since Fraction is immutable
-        def __new__(cls, *args, custom='custom', **kwargs):
-            self = super().__new__(cls, *args, **kwargs)
-            self._custom_attribute = custom
-            return self
-
-        @FractionType.subclass_property
-        def custom_attribute(self):
-            return self._custom_attribute
-
-
-    tree = dict(fraction=MyFraction(7, 9, custom='TESTING!'))
-
-    with pytest.warns(AsdfWarning, match="Failed to add subclass attribute"):
-        with asdf.AsdfFile(tree, extensions=FractionExtension()):
-            pass
-
-
 def test_custom_reference_cycle(tmpdir):
     f1 = FractionWithInverse(3, 5)
     f2 = FractionWithInverse(5, 3)
