@@ -1,5 +1,3 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
 """
 Utility functions for managing tree-like data structures.
 """
@@ -11,7 +9,7 @@ from contextlib import contextmanager
 from . import tagged
 from .exceptions import AsdfWarning
 
-__all__ = ["walk", "iter_tree", "walk_and_modify", "get_children", "is_container"]
+__all__ = ["walk", "iter_tree", "walk_and_modify", "get_children", "is_container", "PendingValue"]
 
 
 def walk(top, callback):
@@ -206,6 +204,19 @@ class _PendingValue:
 PendingValue = _PendingValue()
 
 
+class _RemoveNode:
+    """
+    Class of the RemoveNode singleton instance.  This instance is used
+    as a signal for `asdf.treeutil.walk_and_modify` to remove the
+    node received by the callback.
+    """
+    def __repr__(self):
+        return "RemoveNode"
+
+
+RemoveNode = _RemoveNode()
+
+
 def walk_and_modify(top, callback, ignore_implicit_conversion=False, postorder=True, _context=None):
     """Modify a tree by walking it with a callback function.  It also has
     the effect of doing a deep copy.
@@ -219,11 +230,12 @@ def walk_and_modify(top, callback, ignore_implicit_conversion=False, postorder=T
         A function to call at each node in the tree.  It takes either
         one or two arguments:
 
-        - an instance from the tere
+        - an instance from the tree
         - a json id (optional)
 
         It may return a different instance in order to modify the
-        tree.
+        tree.  If the singleton instance `asdf.treeutil.RemoveNode`
+        is returned, the node will be removed from the tree.
 
         The json id is the context under which any relative URLs
         should be resolved.  It may be `None` if no ids are in the file
@@ -289,7 +301,7 @@ def walk_and_modify(top, callback, ignore_implicit_conversion=False, postorder=T
                 result[key] = PendingValue
             else:
                 value = _recurse(value, json_id)
-                if value is not None:
+                if value is not RemoveNode:
                     result[key] = value
 
         yield result
@@ -299,7 +311,7 @@ def walk_and_modify(top, callback, ignore_implicit_conversion=False, postorder=T
             # be available.
             for key, value in pending_items.items():
                 value = _recurse(value, json_id)
-                if value is not None:
+                if value is not RemoveNode:
                     result[key] = value
                 else:
                     # The callback may have decided to delete
@@ -377,8 +389,10 @@ def walk_and_modify(top, callback, ignore_implicit_conversion=False, postorder=T
         with _context.pending(node):
             # Take note of the "id" field, in case we're modifying
             # a schema and need to know the namespace for resolving
-            # URIs.
-            if isinstance(node, dict) and "id" in node:
+            # URIs.  Ignore an id that is not a string, since it may
+            # be an object defining an id property and not an id
+            # itself (this is common in metaschemas).
+            if isinstance(node, dict) and "id" in node and isinstance(node["id"], str):
                 json_id = node["id"]
 
             if postorder:
