@@ -1,21 +1,37 @@
-# Licensed under a 3-clause BSD style license - see LICENSE.rst
-# -*- coding: utf-8 -*-
-
+import enum
 import inspect
 import math
 import struct
 import types
+import importlib.util
+import re
+from functools import lru_cache
 
-from urllib.parse import urljoin
 from urllib.request import pathname2url
-from urllib import parse as urlparse
 
 import numpy as np
 
+from . import constants
+
+# We're importing our own copy of urllib.parse because
+# we need to patch it to support asdf:// URIs, but it'd
+# be irresponsible to do this for all users of a
+# standard library.
+urllib_parse_spec = importlib.util.find_spec('urllib.parse')
+patched_urllib_parse = importlib.util.module_from_spec(urllib_parse_spec)
+urllib_parse_spec.loader.exec_module(patched_urllib_parse)
+del urllib_parse_spec
+
+# urllib.parse needs to know that it should treat asdf://
+# URIs like http:// URIs for the purposes of joining
+# a relative path to a base URI.
+patched_urllib_parse.uses_relative.append('asdf')
+patched_urllib_parse.uses_netloc.append('asdf')
+
 
 __all__ = ['human_list', 'get_array_base', 'get_base_uri', 'filepath_to_url',
            'iter_subclasses', 'calculate_padding', 'resolve_name', 'NotSet',
-           'is_primitive']
+           'is_primitive', 'uri_match', 'get_class_name']
 
 
 def human_list(l, separator="and"):
@@ -61,15 +77,15 @@ def get_base_uri(uri):
     """
     For a given URI, return the part without any fragment.
     """
-    parts = urlparse.urlparse(uri)
-    return urlparse.urlunparse(list(parts[:5]) + [''])
+    parts = patched_urllib_parse.urlparse(uri)
+    return patched_urllib_parse.urlunparse(list(parts[:5]) + [''])
 
 
 def filepath_to_url(path):
     """
     For a given local file path, return a file:// url.
     """
-    return urljoin('file:', pathname2url(path))
+    return patched_urllib_parse.urljoin('file:', pathname2url(path))
 
 
 def iter_subclasses(cls):
@@ -272,6 +288,14 @@ def resolve_name(name):
     return ret
 
 
+# Kludge to cover up the fact that BuiltinExtension was moved from extension.py
+# to extension/_legacy.py.  Can be removed once BuiltinExtension is dropped
+# in asdf 3.0.
+_CLASS_NAME_OVERRIDES = {
+    "asdf.extension._legacy.BuiltinExtension": "asdf.extension.BuiltinExtension",
+}
+
+
 def get_class_name(obj, instance=True):
     """
     Given a class or instance of a class, returns a string representing the
@@ -285,9 +309,9 @@ def get_class_name(obj, instance=True):
     instance: bool
         Indicates whether given object is an instance of the class to be named
     """
-
     typ = type(obj) if instance else obj
-    return "{}.{}".format(typ.__module__, typ.__name__)
+    class_name = "{}.{}".format(typ.__module__, typ.__qualname__)
+    return _CLASS_NAME_OVERRIDES.get(class_name, class_name)
 
 
 def minversion(module, version, inclusive=True, version_path='__version__'):
@@ -297,7 +321,7 @@ def minversion(module, version, inclusive=True, version_path='__version__'):
 
     By default this uses `pkg_resources.parse_version` to do the version
     comparison if available.  Otherwise it falls back on
-    `distutils.version.LooseVersion`.
+    `packaging.version.Version`.
 
     Parameters
     ----------
@@ -342,7 +366,7 @@ def minversion(module, version, inclusive=True, version_path='__version__'):
     try:
         from pkg_resources import parse_version
     except ImportError:
-        from distutils.version import LooseVersion as parse_version
+        from packaging.version import Version as parse_version
 
     if inclusive:
         return parse_version(have_version) >= parse_version(version)
@@ -430,3 +454,73 @@ def is_primitive(value):
         or isinstance(value, complex)
         or isinstance(value, str)
     )
+
+
+def uri_match(pattern, uri):
+    """
+    Determine if a URI matches a URI pattern with possible
+    wildcards.  The two recognized wildcards:
+
+    "*":  match any character except /
+
+    "**": match any character
+
+    Parameters
+    ----------
+    pattern : str
+        URI pattern.
+    uri : str
+        URI to check against the pattern.
+
+    Returns
+    -------
+    bool
+        `True` if URI matches the pattern.
+    """
+    if not isinstance(uri, str):
+        return False
+
+    if "*" in pattern:
+        return _compile_uri_match_pattern(pattern).fullmatch(uri) is not None
+    else:
+        return pattern == uri
+
+
+@lru_cache(1024)
+def _compile_uri_match_pattern(pattern):
+    # Escape the pattern in case it contains regex special characters
+    # ('.' in particular is common in URIs) and then replace the
+    # escaped asterisks with the appropriate regex matchers.
+    pattern = re.escape(pattern)
+    pattern = pattern.replace(r"\*\*", r".*")
+    pattern = pattern.replace(r"\*", r"[^/]*")
+    return re.compile(pattern)
+
+
+def get_file_type(fd):
+    """
+    Determine the file type of an open GenericFile instance.
+
+    Parameters
+    ----------
+    fd : GenericFile
+
+    Returns
+    -------
+    FileType
+    """
+    if fd.peek(5) == constants.ASDF_MAGIC:
+        return FileType.ASDF
+    elif fd.peek(6) == constants.FITS_MAGIC:
+        return FileType.FITS
+    else:
+        return FileType.UNKNOWN
+
+
+class FileType(enum.Enum):
+    """
+    Enum representing file types recognized by asdf.
+    """
+    ASDF = 1
+    FITS = 2
+    UNKNOWN = 3
